from dcim.models import Interface
from ipam.models import VRF
from utilities.testing import ViewTestCases, create_test_device

from netbox_routing.models import OSPFInstance, OSPFArea, OSPFInterface
from netbox_routing.tests.base import IPAddressFieldMixin

__all__ = (
    'OSPFInstanceTestCase',
    'OSPFAreaTestCase',
    'OSPFInterfaceTestCase',
)


class OSPFInstanceTestCase(
    IPAddressFieldMixin,
    ViewTestCases.GetObjectViewTestCase,
    ViewTestCases.GetObjectChangelogViewTestCase,
    ViewTestCases.CreateObjectViewTestCase,
    ViewTestCases.EditObjectViewTestCase,
    ViewTestCases.DeleteObjectViewTestCase,
    ViewTestCases.ListObjectsViewTestCase,
    ViewTestCases.BulkEditObjectsViewTestCase,
    ViewTestCases.BulkDeleteObjectsViewTestCase,
):
    # ViewTestCases.BulkImportObjectsViewTestCase,
    model = OSPFInstance

    @classmethod
    def setUpTestData(cls):
        vrf = VRF.objects.create(name='Test')
        devices = [
            create_test_device(name='Device 1'),
            create_test_device(name='Device 2'),
        ]

        instances = (
            cls.model(
                name="Instance 0",
                device=devices[0],
                router_id='0.0.0.0',
                process_id='0',
                vrf=None,
            ),
            cls.model(
                name="Instance 1",
                device=devices[0],
                router_id='1.1.1.1',
                process_id='1',
                vrf=None,
            ),
            cls.model(
                name="Instance 2",
                device=devices[0],
                router_id='2.2.2.2',
                process_id='2',
                vrf=vrf,
            ),
        )
        cls.model.objects.bulk_create(instances)

        cls.form_data = {
            'name': 'Instance X',
            'device': devices[1].pk,
            'router_id': '4.4.4.4',
            'process_id': 4,
            'vrf': vrf.pk,
        }

        cls.bulk_edit_data = {
            'description': 'A test Instance description',
            'vrf': vrf.pk,
        }

    def _get_base_url(self):
        return 'plugins:netbox_routing:ospfinstance_{}'


class OSPFAreaTestCase(
    IPAddressFieldMixin,
    ViewTestCases.GetObjectViewTestCase,
    ViewTestCases.GetObjectChangelogViewTestCase,
    ViewTestCases.CreateObjectViewTestCase,
    ViewTestCases.EditObjectViewTestCase,
    ViewTestCases.DeleteObjectViewTestCase,
    ViewTestCases.ListObjectsViewTestCase,
    ViewTestCases.BulkEditObjectsViewTestCase,
    ViewTestCases.BulkDeleteObjectsViewTestCase,
):
    # ViewTestCases.BulkImportObjectsViewTestCase,
    model = OSPFArea

    @classmethod
    def setUpTestData(cls):
        areas = (
            cls.model(area_id='0.0.0.0', area_type='stub'),
            cls.model(area_id='1.1.1.1', area_type='tsa'),
            cls.model(area_id='2.2.2.2', area_type='nssa'),
        )
        cls.model.objects.bulk_create(areas)

        cls.form_data = {
            'area_id': '4.4.4.4',
            'area_type': 'standard',
        }

        cls.bulk_edit_data = {
            'description': 'A test Area description',
            'area_type': 'backbone',
        }

    def _get_base_url(self):
        return 'plugins:netbox_routing:ospfarea_{}'


class OSPFInterfaceTestCase(
    IPAddressFieldMixin,
    ViewTestCases.GetObjectViewTestCase,
    ViewTestCases.GetObjectChangelogViewTestCase,
    ViewTestCases.CreateObjectViewTestCase,
    ViewTestCases.EditObjectViewTestCase,
    ViewTestCases.DeleteObjectViewTestCase,
    ViewTestCases.ListObjectsViewTestCase,
    ViewTestCases.BulkEditObjectsViewTestCase,
    ViewTestCases.BulkDeleteObjectsViewTestCase,
):
    # ViewTestCases.BulkImportObjectsViewTestCase,
    model = OSPFInterface

    @classmethod
    def setUpTestData(cls):
        devices = [
            create_test_device(name='Device 1'),
            create_test_device(name='Device 2'),
        ]
        interfaces = (
            Interface(name='Interface 1', device=devices[0], type='virtual'),
            Interface(name='Interface 2', device=devices[0], type='virtual'),
            Interface(name='Interface 3', device=devices[1], type='virtual'),
            Interface(name='Interface 4', device=devices[1], type='virtual'),
        )
        Interface.objects.bulk_create(interfaces)

        instances = (
            OSPFInstance(
                name="Instance 0",
                device=devices[0],
                router_id='0.0.0.0',
                process_id='0',
            ),
            OSPFInstance(
                name="Instance 1",
                device=devices[0],
                router_id='1.1.1.1',
                process_id='1',
            ),
            OSPFInstance(
                name="Instance 2",
                device=devices[1],
                router_id='2.2.2.2',
                process_id='2',
            ),
            OSPFInstance(
                name="Instance 3",
                device=devices[1],
                router_id='3.3.3.3',
                process_id='3',
            ),
        )
        OSPFInstance.objects.bulk_create(instances)

        areas = (
            OSPFArea(area_id='0.0.0.0'),
            OSPFArea(area_id='1.1.1.1'),
            OSPFArea(area_id='2.2.2.2'),
            OSPFArea(area_id='3.3.3.3'),
        )
        OSPFArea.objects.bulk_create(areas)

        ospfinterfaces = (
            cls.model(
                interface=interfaces[0],
                instance=instances[0],
                area=areas[0],
                passive=False,
            ),
            cls.model(interface=interfaces[1], instance=instances[1], area=areas[1]),
            cls.model(
                interface=interfaces[2],
                instance=instances[2],
                area=areas[2],
                passive=False,
            ),
        )
        cls.model.objects.bulk_create(ospfinterfaces)

        cls.form_data = {
            'interface': interfaces[3].pk,
            'area': areas[3].pk,
            'instance': instances[3].pk,
            'passive': True,
        }

        cls.bulk_edit_data = {
            'description': 'A test Interface description',
            'passive': False,
        }

    def _get_base_url(self):
        return 'plugins:netbox_routing:ospfinterface_{}'



from .static import *
from .objects import *
from .community import *
from .ospf import *
from .eigrp import *
from .bgp import *
from .core import *

__all__ = (
    # Core View Extensions
    'DeviceStaticRoutesView',
    # Static
    'StaticRouteListView',
    'StaticRouteView',
    'StaticRouteDevicesView',
    'StaticRouteEditView',
    'StaticRouteBulkEditView',
    'StaticRouteDeleteView',
    'StaticRouteBulkDeleteView',
    # OSPF
    'OSPFInstanceListView',
    'OSPFInstanceView',
    'OSPFInstanceEditView',
    'OSPFInstanceDeleteView',
    'OSPFInstanceInterfacesView',
    'OSPFAreaListView',
    'OSPFAreaView',
    'OSPFAreaInterfacesView',
    'OSPFAreaEditView',
    'OSPFAreaDeleteView',
    'OSPFInterfaceListView',
    'OSPFInterfaceView',
    'OSPFInterfaceEditView',
    'OSPFInterfaceDeleteView',
    # EIGRP
    'EIGRPRouterListView',
    'EIGRPRouterView',
    'EIGRPRouterInterfacesView',
    'EIGRPRouterEditView',
    'EIGRPRouterImportView',
    'EIGRPRouterBulkEditView',
    'EIGRPRouterDeleteView',
    'EIGRPRouterBulkDeleteView',
    'EIGRPAddressFamilyListView',
    'EIGRPAddressFamilyView',
    'EIGRPAddressFamilyInterfacesView',
    'EIGRPAddressFamilyEditView',
    'EIGRPAddressFamilyBulkEditView',
    'EIGRPAddressFamilyDeleteView',
    'EIGRPAddressFamilyBulkDeleteView',
    'EIGRPNetworkListView',
    'EIGRPNetworkView',
    'EIGRPNetworkEditView',
    'EIGRPNetworkBulkEditView',
    'EIGRPNetworkDeleteView',
    'EIGRPNetworkBulkDeleteView',
    'EIGRPInterfaceListView',
    'EIGRPInterfaceView',
    'EIGRPInterfaceEditView',
    'EIGRPInterfaceBulkEditView',
    'EIGRPInterfaceDeleteView',
    'EIGRPInterfaceBulkDeleteView',
    'BGPRouterView',
    'BGPRouterEditView',
    # Routing Objects
    'CommunityListListView',
    'CommunityListView',
    # 'CommunityListEntriesView',
    'CommunityListEditView',
    'CommunityListDeleteView',
    'CommunityListEntryListView',
    'CommunityListEntryDetailView',
    'CommunityListEntryEditView',
    'CommunityListEntryDeleteView',
    'CommunityListView',
    'CommunityView',
    'CommunityEditView',
    'CommunityDeleteView',
    'PrefixListListView',
    'PrefixListView',
    'PrefixListEntriesView',
    'PrefixListEditView',
    'PrefixListDeleteView',
    'PrefixListEntryListView',
    'PrefixListEntryView',
    'PrefixListEntryEditView',
    'PrefixListEntryDeleteView',
    'PrefixListEntryBulkEditView',
    'PrefixListEntryBulkDeleteView',
    'RouteMapListView',
    'RouteMapView',
    'RouteMapEntriesView',
    'RouteMapEditView',
    'RouteMapDeleteView',
    'RouteMapEntryListView',
    'RouteMapEntryView',
    'RouteMapEntryEditView',
    'RouteMapEntryDeleteView',
    'RouteMapEntryBulkEditView',
    'RouteMapEntryBulkDeleteView',
)



from django.test import TestCase

from dcim.models import Device, Interface
from ipam.models import VRF
from utilities.testing import create_test_device

from netbox_routing.filtersets import *
from netbox_routing.models import *

__all__ = (
    'OSPFInstanceTestCase',
    'OSPFAreaTestCase',
    'OSPFInterfaceTestCase',
)


class OSPFInstanceTestCase(TestCase):
    queryset = OSPFInstance.objects.all()
    filterset = OSPFInstanceFilterSet

    @classmethod
    def setUpTestData(cls):
        vrfs = (
            VRF(name='VRF 1'),
            VRF(name='VRF 2'),
            VRF(name='VRF 3'),
        )
        VRF.objects.bulk_create(vrfs)

        devices = [
            create_test_device(name='Device 1'),
            create_test_device(name='Device 2'),
            create_test_device(name='Device 3'),
            create_test_device(name='Device 4'),
        ]

        data = (
            OSPFInstance(
                name='Instance 1',
                device=devices[0],
                router_id='0.0.0.0',
                process_id=0,
                vrf=vrfs[0],
            ),
            OSPFInstance(
                name='Instance 2',
                device=devices[1],
                router_id='1.1.1.1',
                process_id=1,
                vrf=vrfs[1],
            ),
            OSPFInstance(
                name='Instance 3',
                device=devices[2],
                router_id='2.2.2.2',
                process_id=2,
                vrf=None,
            ),
            OSPFInstance(
                name='Instance 3',
                device=devices[3],
                router_id='3.3.3.3',
                process_id=3,
                vrf=vrfs[2],
            ),
        )

        OSPFInstance.objects.bulk_create(data)

    def test_q(self):
        params = {'q': 'Instance 1'}
        self.assertEqual(self.filterset(params, self.queryset).qs.count(), 1)

    def test_name(self):
        params = {'name': ['Instance 1', 'Instance 2']}
        self.assertEqual(self.filterset(params, self.queryset).qs.count(), 2)

    def test_vrf(self):
        data = VRF.objects.all()[0:2]

        params = {'vrf_id': [data[0].pk, data[1].pk]}
        self.assertEqual(self.filterset(params, self.queryset).qs.count(), 2)

        params = {'vrf': [data[0].name, data[1].name]}
        self.assertEqual(self.filterset(params, self.queryset).qs.count(), 2)

    def test_device(self):
        data = Device.objects.all()[0:2]

        params = {'device_id': [data[0].pk, data[1].pk]}
        self.assertEqual(self.filterset(params, self.queryset).qs.count(), 2)

        params = {'device': [data[0].name, data[1].name]}
        self.assertEqual(self.filterset(params, self.queryset).qs.count(), 2)

    def test_process_id(self):
        params = {'process_id': [1, 2]}
        self.assertEqual(self.filterset(params, self.queryset).qs.count(), 2)

    def test_router_id(self):
        params = {'router_id': ['1.1.1.1', '2.2.2.2']}
        self.assertEqual(self.filterset(params, self.queryset).qs.count(), 2)


class OSPFAreaTestCase(TestCase):
    queryset = OSPFArea.objects.all()
    filterset = OSPFAreaFilterSet

    @classmethod
    def setUpTestData(cls):

        data = (
            OSPFArea(area_id='0.0.0.0'),
            OSPFArea(area_id='1.1.1.1'),
            OSPFArea(area_id=0),
            OSPFArea(area_id=1),
        )

        OSPFArea.objects.bulk_create(data)

    def test_q(self):
        params = {'q': '1.1.1.1'}
        self.assertEqual(self.filterset(params, self.queryset).qs.count(), 1)

    def test_area_id(self):
        params = {'area_id': ['0', '1.1.1.1']}
        self.assertEqual(self.filterset(params, self.queryset).qs.count(), 2)


class OSPFInterfaceTestCase(TestCase):
    queryset = OSPFInterface.objects.all()
    filterset = OSPFInterfaceFilterSet

    @classmethod
    def setUpTestData(cls):
        vrfs = (
            VRF(name='VRF 1'),
            VRF(name='VRF 2'),
            VRF(name='VRF 3'),
        )
        VRF.objects.bulk_create(vrfs)

        devices = (
            create_test_device(name='Device 1'),
            create_test_device(name='Device 2'),
            create_test_device(name='Device 3'),
            create_test_device(name='Device 4'),
        )
        interfaces = (
            Interface(device=devices[0], name='Interface 1', type='virtual'),
            Interface(device=devices[1], name='Interface 2', type='virtual'),
            Interface(device=devices[2], name='Interface 3', type='virtual'),
            Interface(device=devices[3], name='Interface 4', type='virtual'),
        )
        Interface.objects.bulk_create(interfaces)

        areas = (
            OSPFArea(area_id='0.0.0.0'),
            OSPFArea(area_id=1),
            OSPFArea(area_id='2.2.2.2'),
            OSPFArea(area_id=3),
        )
        OSPFArea.objects.bulk_create(areas)

        instances = (
            OSPFInstance(
                name='Instance 0',
                device=devices[0],
                router_id='0.0.0.0',
                process_id=0,
                vrf=vrfs[0],
            ),
            OSPFInstance(
                name='Instance 1',
                device=devices[1],
                router_id='1.1.1.1',
                process_id=1,
                vrf=vrfs[1],
            ),
            OSPFInstance(
                name='Instance 2',
                device=devices[2],
                router_id='2.2.2.2',
                process_id=2,
                vrf=None,
            ),
            OSPFInstance(
                name='Instance 3',
                device=devices[3],
                router_id='3.3.3.3',
                process_id=3,
                vrf=vrfs[2],
            ),
        )
        OSPFInstance.objects.bulk_create(instances)

        data = (
            OSPFInterface(
                interface=interfaces[0],
                instance=instances[0],
                area=areas[0],
                passive=True,
            ),
            OSPFInterface(
                interface=interfaces[1],
                instance=instances[1],
                area=areas[1],
                passive=False,
            ),
            OSPFInterface(
                interface=interfaces[2],
                instance=instances[2],
                area=areas[2],
                passive=True,
            ),
            OSPFInterface(
                interface=interfaces[3],
                instance=instances[3],
                area=areas[3],
                passive=False,
            ),
        )
        OSPFInterface.objects.bulk_create(data)

    def test_q(self):
        params = {'q': 'Interface 1'}
        self.assertEqual(self.filterset(params, self.queryset).qs.count(), 1)

        params = {'q': 'Instance 1'}
        self.assertEqual(self.filterset(params, self.queryset).qs.count(), 1)

    def test_instances(self):
        data = OSPFInstance.objects.all()[0:2]

        params = {'instance_id': [data[0].pk, data[1].pk]}
        self.assertEqual(self.filterset(params, self.queryset).qs.count(), 2)

        params = {'instance': [data[0].name, data[1].name]}
        self.assertEqual(self.filterset(params, self.queryset).qs.count(), 2)

    def test_area(self):
        data = OSPFArea.objects.all()[0:2]

        params = {'area_id': [data[0].pk, data[1].pk]}
        self.assertEqual(self.filterset(params, self.queryset).qs.count(), 2)

        params = {'area': [data[0].area_id, data[1].area_id]}
        self.assertEqual(self.filterset(params, self.queryset).qs.count(), 2)

    def test_vrfs(self):
        data = VRF.objects.all()[0:2]

        params = {'vrf_id': [data[0].pk, data[1].pk]}
        self.assertEqual(self.filterset(params, self.queryset).qs.count(), 2)

        params = {'vrf': [data[0].name, data[1].name]}
        self.assertEqual(self.filterset(params, self.queryset).qs.count(), 2)

    def test_devices(self):
        data = Device.objects.all()[0:2]

        params = {'device_id': [data[0].pk, data[1].pk]}
        self.assertEqual(self.filterset(params, self.queryset).qs.count(), 2)

        params = {'device': [data[0].name, data[1].name]}
        self.assertEqual(self.filterset(params, self.queryset).qs.count(), 2)

    def test_interfaces(self):
        data = Interface.objects.all()[0:2]

        params = {'interface_id': [data[0].pk, data[1].pk]}
        self.assertEqual(self.filterset(params, self.queryset).qs.count(), 2)

        params = {'interface': [data[0].name, data[1].name]}
        self.assertEqual(self.filterset(params, self.queryset).qs.count(), 2)

    def test_passive(self):
        params = {'passive': True}
        self.assertEqual(self.filterset(params, self.queryset).qs.count(), 2)



from django.db import models
from taggit.managers import TaggableManager

from ipam.fields import IPNetworkField
from netbox.models import NetBoxModel


class AbsoluteURLMixin:
    def get_absolute_url(self):
        return None


class ASPathList(AbsoluteURLMixin, NetBoxModel):
    """
    as-path access list, as-path filter
    """

    name = models.CharField(max_length=100)
    description = models.CharField(max_length=200, blank=True)
    comments = models.TextField(blank=True)

    class Meta:
        app_label = 'netbox_bgp'
        managed = False

    def __str__(self):
        return self.name


class TagMixin:
    tags = TaggableManager(
        through='extras.TaggedItem', ordering=('weight', 'name'), related_name='+'
    )


class ASPathListRule(AbsoluteURLMixin, NetBoxModel):
    aspath_list = models.ForeignKey(
        to=ASPathList, on_delete=models.CASCADE, related_name='aspathlistrules'
    )
    index = models.PositiveIntegerField()
    action = models.CharField(max_length=30)
    pattern = models.CharField(
        max_length=200,
    )
    description = models.CharField(max_length=200, blank=True)
    comments = models.TextField(blank=True)

    def __str__(self):
        return f'{self.aspath_list}: {self.action} {self.pattern}'

    class Meta:
        app_label = 'netbox_bgp'
        managed = False


class RoutingPolicy(AbsoluteURLMixin, NetBoxModel):
    name = models.CharField(max_length=100)
    description = models.CharField(max_length=200, blank=True)
    comments = models.TextField(blank=True)
    weight = models.PositiveIntegerField(blank=True, null=True)

    def __str__(self):
        return self.name

    class Meta:
        app_label = 'netbox_bgp'
        managed = False


class BGPPeerGroup(AbsoluteURLMixin, NetBoxModel):
    """ """

    name = models.CharField(max_length=100)
    description = models.CharField(max_length=200, blank=True)
    comments = models.TextField(blank=True)

    class Meta:
        verbose_name_plural = 'Peer Groups'
        unique_together = ['name', 'description']
        ordering = ['name']
        app_label = 'netbox_bgp'
        managed = False

    def __str__(self):
        return self.name


class BGPBase(AbsoluteURLMixin, NetBoxModel):
    """ """

    site = models.ForeignKey(
        to='dcim.Site',
        on_delete=models.PROTECT,
        related_name="+",
        related_query_name="+",
        blank=True,
        null=True,
    )
    tenant = models.ForeignKey(
        to='tenancy.Tenant', on_delete=models.PROTECT, blank=True, null=True
    )
    status = models.CharField(max_length=50)
    role = models.ForeignKey(
        to='ipam.Role', on_delete=models.SET_NULL, blank=True, null=True
    )
    description = models.CharField(max_length=200, blank=True)
    comments = models.TextField(blank=True)

    class Meta:
        abstract = True


class Community(BGPBase):
    """ """

    value = models.CharField(max_length=64)

    class Meta:
        verbose_name_plural = 'Communities'
        ordering = ['value']
        app_label = 'netbox_bgp'
        managed = False

    def __str__(self):
        return self.value


class CommunityList(AbsoluteURLMixin, NetBoxModel):
    """ """

    name = models.CharField(max_length=100)
    description = models.CharField(max_length=200, blank=True)
    comments = models.TextField(blank=True)

    class Meta:
        verbose_name_plural = 'Community Lists'
        unique_together = ['name', 'description']
        ordering = ['name']
        app_label = 'netbox_bgp'
        managed = False

    def __str__(self):
        return self.name


class CommunityListRule(AbsoluteURLMixin, NetBoxModel):
    """ """

    community_list = models.ForeignKey(
        to=CommunityList, on_delete=models.CASCADE, related_name='commlistrules'
    )
    action = models.CharField(max_length=30)
    community = models.ForeignKey(
        to=Community,
        related_name='+',
        on_delete=models.CASCADE,
    )
    description = models.CharField(max_length=200, blank=True)
    comments = models.TextField(blank=True)

    def __str__(self):
        return f'{self.community_list}: {self.action} {self.community}'

    class Meta:
        ordering = ['community_list', 'community']
        app_label = 'netbox_bgp'
        managed = False


class PrefixList(AbsoluteURLMixin, NetBoxModel):
    """ """

    name = models.CharField(max_length=100)
    description = models.CharField(max_length=200, blank=True)
    family = models.CharField(max_length=10)
    comments = models.TextField(blank=True)

    class Meta:
        verbose_name_plural = 'Prefix Lists'
        unique_together = ['name', 'description', 'family']
        ordering = ['name']
        app_label = 'netbox_bgp'
        managed = False

    def __str__(self):
        return self.name


class PrefixListRule(AbsoluteURLMixin, NetBoxModel):
    """ """

    prefix_list = models.ForeignKey(
        to=PrefixList, on_delete=models.CASCADE, related_name='prefrules'
    )
    index = models.PositiveIntegerField()
    action = models.CharField(max_length=30)
    prefix = models.ForeignKey(
        to='ipam.Prefix',
        blank=True,
        null=True,
        related_name="+",
        related_query_name="+",
        on_delete=models.CASCADE,
    )
    prefix_custom = IPNetworkField(
        blank=True,
        null=True,
    )
    ge = models.PositiveSmallIntegerField(blank=True, null=True)
    le = models.PositiveSmallIntegerField(blank=True, null=True)
    description = models.CharField(max_length=200, blank=True)
    comments = models.TextField(blank=True)

    class Meta:
        unique_together = ('prefix_list', 'index')
        ordering = ['prefix_list', 'index']
        app_label = 'netbox_bgp'
        managed = False

    def __str__(self):
        return f'{self.prefix_list}: Rule {self.index}'


class BGPSession(AbsoluteURLMixin, NetBoxModel):
    name = models.CharField(max_length=256, blank=True, null=True)
    site = models.ForeignKey(
        to='dcim.Site', on_delete=models.SET_NULL, blank=True, null=True
    )
    tenant = models.ForeignKey(
        to='tenancy.Tenant', on_delete=models.PROTECT, blank=True, null=True
    )
    device = models.ForeignKey(
        to='dcim.Device',
        on_delete=models.PROTECT,
        null=True,
        blank=True,
    )

    virtualmachine = models.ForeignKey(
        to='virtualization.VirtualMachine',
        on_delete=models.PROTECT,
        null=True,
        blank=True,
    )

    local_address = models.ForeignKey(
        to='ipam.IPAddress',
        on_delete=models.PROTECT,
        related_name="+",
        related_query_name="+",
    )
    remote_address = models.ForeignKey(
        to='ipam.IPAddress',
        on_delete=models.PROTECT,
        related_name="+",
        related_query_name="+",
    )
    local_as = models.ForeignKey(
        to='ipam.ASN',
        on_delete=models.PROTECT,
        related_name='+',
        related_query_name='+',
    )
    remote_as = models.ForeignKey(
        to='ipam.ASN',
        on_delete=models.PROTECT,
        related_name="+",
        related_query_name="+",
    )
    status = models.CharField(max_length=50)
    description = models.CharField(max_length=200, blank=True)
    peer_group = models.ForeignKey(
        BGPPeerGroup, on_delete=models.SET_NULL, blank=True, null=True
    )
    prefix_list_in = models.ForeignKey(
        to=PrefixList,
        blank=True,
        null=True,
        on_delete=models.SET_NULL,
        related_name='session_prefix_in',
    )
    prefix_list_out = models.ForeignKey(
        to=PrefixList,
        blank=True,
        null=True,
        on_delete=models.SET_NULL,
        related_name='session_prefix_out',
    )
    comments = models.TextField(blank=True)

    class Meta:
        verbose_name_plural = 'BGP Sessions'
        unique_together = [
            ['device', 'local_address', 'local_as', 'remote_address', 'remote_as'],
            [
                'virtualmachine',
                'local_address',
                'local_as',
                'remote_address',
                'remote_as',
            ],
        ]
        ordering = ('name', 'pk')  # Name may be null
        app_label = 'netbox_bgp'
        managed = False

    @property
    def label(self):
        """
        Return the session name if set; otherwise return a generated name if available.
        """
        if self.name:
            return self.name
        return f'{self.remote_address}:{self.remote_as}'

    def __str__(self):
        if self.device:
            return f'{self.device}:{self.label}'
        elif self.virtualmachine:
            return f'{self.virtualmachine}:{self.label}'
        else:
            return f'{self.label}'


class RoutingPolicyRule(AbsoluteURLMixin, NetBoxModel):
    routing_policy = models.ForeignKey(
        to=RoutingPolicy, on_delete=models.CASCADE, related_name='rules'
    )
    index = models.PositiveIntegerField()
    action = models.CharField(max_length=30)
    description = models.CharField(max_length=500, blank=True)
    continue_entry = models.PositiveIntegerField(blank=True, null=True)
    match_custom = models.JSONField(
        blank=True,
        null=True,
    )
    set_actions = models.JSONField(
        blank=True,
        null=True,
    )
    comments = models.TextField(blank=True)

    def __str__(self):
        return f'{self.routing_policy}: Rule {self.index}'

    class Meta:
        ordering = ['routing_policy', 'index']
        unique_together = ('routing_policy', 'index')
        ordering = ['routing_policy', 'index']
        app_label = 'netbox_bgp'
        managed = False


class BGPPeerGroup_ImportPolicies(models.Model):

    bgppeergroup = models.ForeignKey(
        to='BGPPeerGroup',
        on_delete=models.CASCADE,
        related_name="import_policies",
        related_query_name="import_policies",
    )

    routingpolicy = models.ForeignKey(
        to='RoutingPolicy',
        on_delete=models.CASCADE,
        related_name="+",
        related_query_name="+",
    )

    class Meta:
        app_label = 'netbox_bgp'
        db_table = 'netbox_bgp_bgppeergroup_import_policies'
        managed = False


class BGPPeerGroup_ExportPolicies(models.Model):

    bgppeergroup = models.ForeignKey(
        to='BGPPeerGroup',
        on_delete=models.CASCADE,
        related_name="export_policies",
        related_query_name="export_policies",
    )

    routingpolicy = models.ForeignKey(
        to='RoutingPolicy',
        on_delete=models.CASCADE,
        related_name="group_export_policies",
        related_query_name="group_export_policies",
    )

    class Meta:
        app_label = 'netbox_bgp'
        db_table = 'netbox_bgp_bgppeergroup_export_policies'
        managed = False


class BGPSession_ImportPolicies(models.Model):

    bgpsession = models.ForeignKey(
        to='BGPSession',
        on_delete=models.CASCADE,
        related_name="import_policies",
        related_query_name="import_policies",
    )

    routingpolicy = models.ForeignKey(
        to='RoutingPolicy',
        on_delete=models.CASCADE,
        related_name="group_import_policies",
        related_query_name="group_import_policies",
    )

    class Meta:
        app_label = 'netbox_bgp'
        db_table = 'netbox_bgp_bgpsession_import_policies'
        managed = False


class BGPSession_ExportPolicies(models.Model):

    bgpsession = models.ForeignKey(
        to='BGPSession',
        on_delete=models.CASCADE,
        related_name="export_policies",
        related_query_name="export_policies",
    )

    routingpolicy = models.ForeignKey(
        to='RoutingPolicy',
        on_delete=models.CASCADE,
        related_name="session_export_policies",
        related_query_name="session_export_policies",
    )

    class Meta:
        app_label = 'netbox_bgp'
        db_table = 'netbox_bgp_bgpsession_export_policies'
        managed = False


class RoutingPolicyRule_Match_Community(models.Model):

    routingpolicyrule = models.ForeignKey(
        to='RoutingPolicyRule',
        on_delete=models.CASCADE,
        related_name="match_community",
        related_query_name="match_community",
    )

    community = models.ForeignKey(
        to='Community',
        on_delete=models.CASCADE,
        related_name="+",
        related_query_name="+",
    )

    class Meta:
        app_label = 'netbox_bgp'
        db_table = 'netbox_bgp_routingpolicyrule_match_community'
        managed = False


class RoutingPolicyRule_Match_CommunityList(models.Model):

    routingpolicyrule = models.ForeignKey(
        to='RoutingPolicyRule',
        on_delete=models.CASCADE,
        related_name="match_community_list",
        related_query_name="match_community_list",
    )

    communitylist = models.ForeignKey(
        to='CommunityList',
        on_delete=models.CASCADE,
        related_name="+",
        related_query_name="+",
    )

    class Meta:
        app_label = 'netbox_bgp'
        db_table = 'netbox_bgp_routingpolicyrule_match_community_list'
        managed = False


class RoutingPolicyRule_Match_ASPathList(models.Model):

    routingpolicyrule = models.ForeignKey(
        to='RoutingPolicyRule',
        on_delete=models.CASCADE,
        related_name="match_aspath_list",
        related_query_name="match_aspath_list",
    )

    aspathlist = models.ForeignKey(
        to='ASPathList',
        on_delete=models.CASCADE,
        related_name="+",
        related_query_name="+",
    )

    class Meta:
        app_label = 'netbox_bgp'
        db_table = 'netbox_bgp_routingpolicyrule_match_aspath_list'
        managed = False


class RoutingPolicyRule_Match_IPAddress(models.Model):

    routingpolicyrule = models.ForeignKey(
        to='RoutingPolicyRule',
        on_delete=models.CASCADE,
        related_name="match_ip_address",
        related_query_name="match_ip_address",
    )

    prefixlist = models.ForeignKey(
        to='PrefixList',
        on_delete=models.CASCADE,
        related_name="+",
        related_query_name="+",
    )

    class Meta:
        app_label = 'netbox_bgp'
        db_table = 'netbox_bgp_routingpolicyrule_match_ip_address'
        managed = False


class RoutingPolicyRule_Match_IPV6Address(models.Model):

    routingpolicyrule = models.ForeignKey(
        to='RoutingPolicyRule',
        on_delete=models.CASCADE,
        related_name="match_ipv6_address",
        related_query_name="match_ipv6_address",
    )

    prefixlist = models.ForeignKey(
        to='PrefixList',
        on_delete=models.CASCADE,
        related_name="+",
        related_query_name="+",
    )

    class Meta:
        app_label = 'netbox_bgp'
        db_table = 'netbox_bgp_routingpolicyrule_match_ipv6_address'
        managed = False



from typing import Annotated

import strawberry
import strawberry_django

from netbox.graphql.types import PrimaryObjectType

from netbox_routing import models
from netbox_routing.graphql.community.filters import (
    CommunityFilter,
    CommunityListFilter,
    CommunityListEntryFilter,
)

__all__ = (
    'CommunityType',
    'CommunityListType',
    'CommunityListEntryType',
)


@strawberry_django.type(models.Community, fields='__all__', filters=CommunityFilter)
class CommunityType(PrimaryObjectType):

    community: str
    status: str
    role: (
        Annotated["RoleType", strawberry.lazy('ipam.graphql.types')] | None
    )  # noqa: F821
    tenant: (
        Annotated["TenantType", strawberry.lazy('tenancy.graphql.types')] | None
    )  # noqa: F821


@strawberry_django.type(
    models.CommunityList, fields='__all__', filters=CommunityListFilter
)
class CommunityListType(PrimaryObjectType):

    name: str
    tenant: Annotated["TenantType", strawberry.lazy('tenancy.graphql.types')] | None


@strawberry_django.type(
    models.CommunityListEntry, fields='__all__', filters=CommunityListEntryFilter
)
class CommunityListEntryType(PrimaryObjectType):

    community_list: Annotated[
        "CommunityListType", strawberry.lazy('netbox_routing.graphql.types')
    ]
    community: Annotated[
        "CommunityType", strawberry.lazy('netbox_routing.graphql.types')
    ]



from netbox.api.viewsets import NetBoxModelViewSet
from netbox_routing import filtersets
from netbox_routing.api._serializers.bgp import *
from netbox_routing.models.bgp import *

__all__ = (
    'BGPRouterViewSet',
    'BGPScopeViewSet',
    'BGPAddressFamilyViewSet',
    'BGPSettingViewSet',
    'BGPPeerViewSet',
    'BGPPeerAddressFamilyViewSet',
    'BGPPeerTemplateViewSet',
    'BGPPolicyTemplateViewSet',
    'BGPSessionTemplateViewSet',
    'BFDProfileViewSet',
)


class BGPSettingViewSet(NetBoxModelViewSet):
    queryset = BGPSetting.objects.all()
    serializer_class = BGPSettingSerializer
    filterset_class = filtersets.BGPSettingFilterSet


class BGPRouterViewSet(NetBoxModelViewSet):
    queryset = BGPRouter.objects.all()
    serializer_class = BGPRouterSerializer
    filterset_class = filtersets.BGPRouterFilterSet


class BGPScopeViewSet(NetBoxModelViewSet):
    queryset = BGPScope.objects.all()
    serializer_class = BGPScopeSerializer
    filterset_class = filtersets.BGPScopeFilterSet


class BGPAddressFamilyViewSet(NetBoxModelViewSet):
    queryset = BGPAddressFamily.objects.all()
    serializer_class = BGPAddressFamilySerializer
    filterset_class = filtersets.BGPAddressFamilyFilterSet


class BGPPeerViewSet(NetBoxModelViewSet):
    queryset = BGPPeer.objects.all()
    serializer_class = BGPPeerSerializer
    filterset_class = filtersets.BGPPeerFilterSet


class BGPPeerTemplateViewSet(NetBoxModelViewSet):
    queryset = BGPPeerTemplate.objects.all()
    serializer_class = BGPPeerTemplateSerializer
    filterset_class = filtersets.BGPPeerTemplateFilterSet


class BGPPolicyTemplateViewSet(NetBoxModelViewSet):
    queryset = BGPPolicyTemplate.objects.all()
    serializer_class = BGPPolicyTemplateSerializer
    filterset_class = filtersets.BGPPolicyTemplateFilterSet


class BGPSessionTemplateViewSet(NetBoxModelViewSet):
    queryset = BGPSessionTemplate.objects.all()
    serializer_class = BGPSessionTemplateSerializer
    filterset_class = filtersets.BGPSessionTemplateFilterSet


class BGPPeerAddressFamilyViewSet(NetBoxModelViewSet):
    queryset = BGPPeerAddressFamily.objects.all()
    serializer_class = BGPPeerAddressFamilySerializer
    filterset_class = filtersets.BGPPeerAddressFamilyFilterSet


class BFDProfileViewSet(NetBoxModelViewSet):
    queryset = BFDProfile.objects.all()
    serializer_class = BFDProfileSerializer
    filterset_class = filtersets.BFDProfileFilterSet



from netbox.search import SearchIndex, register_search
from netbox_routing.models.ospf import OSPFInstance, OSPFArea

__all__ = (
    'OSPFInstanceIndex',
    'OSPFAreaIndex',
)


@register_search
class OSPFInstanceIndex(SearchIndex):
    model = OSPFInstance
    fields = (
        ('name', 100),
        ('router_id', 200),
        ('process_id', 300),
        ('comments', 5000),
    )
    display_attrs = ('process_id', 'device', 'vrf')


@register_search
class OSPFAreaIndex(SearchIndex):
    model = OSPFArea
    fields = (
        ('area_id', 100),
        ('comments', 5000),
    )



from rest_framework import serializers

from ipam.api.serializers_.roles import RoleSerializer
from netbox.api.serializers import NetBoxModelSerializer
from tenancy.api.serializers_.tenants import TenantSerializer

from netbox_routing.models.community import *

__all__ = (
    'CommunityListSerializer',
    'CommunitySerializer',
    'CommunityListEntrySerializer',
)


class CommunitySerializer(NetBoxModelSerializer):
    url = serializers.HyperlinkedIdentityField(
        view_name='plugins-api:netbox_routing-api:community-detail'
    )
    role = RoleSerializer(nested=True, required=False)
    tenant = TenantSerializer(nested=True, required=False)

    class Meta:
        model = Community
        fields = (
            'url',
            'id',
            'display',
            'community',
            'role',
            'status',
            'tenant',
            'description',
            'comments',
        )
        brief_fields = ('url', 'id', 'display', 'community', 'description')


class CommunityListSerializer(NetBoxModelSerializer):
    url = serializers.HyperlinkedIdentityField(
        view_name='plugins-api:netbox_routing-api:communitylist-detail'
    )
    tenant = TenantSerializer(nested=True, required=False)

    class Meta:
        model = CommunityList
        fields = (
            'url',
            'id',
            'display',
            'name',
            'tenant',
            'description',
            'comments',
        )
        brief_fields = ('url', 'id', 'display', 'name', 'description')


class CommunityListEntrySerializer(NetBoxModelSerializer):
    url = serializers.HyperlinkedIdentityField(
        view_name='plugins-api:netbox_routing-api:communitylistentry-detail'
    )

    class Meta:
        model = CommunityListEntry
        fields = (
            'url',
            'id',
            'display',
            'community_list',
            'action',
            'community',
            'description',
            'comments',
        )
        brief_fields = (
            'url',
            'id',
            'display',
            'community_list',
            'action',
            'community',
            'description',
        )



from netaddr.ip import IPAddress
from ipam.models import VRF
from utilities.testing import APIViewTestCases, create_test_device

from netbox_routing.models import StaticRoute
from netbox_routing.tests.base import IPAddressFieldMixin

__all__ = ('StaticRouteTestCase',)


class StaticRouteTestCase(IPAddressFieldMixin, APIViewTestCases.APIViewTestCase):
    model = StaticRoute
    view_namespace = "plugins-api:netbox_routing"
    brief_fields = [
        'description',
        'display',
        'id',
        'name',
        'next_hop',
        'prefix',
        'url',
    ]

    bulk_update_data = {'metric': 5}

    @classmethod
    def setUpTestData(cls):

        device = create_test_device(name='Test Device')
        vrf = VRF.objects.create(name='Test VRF')

        nh = IPAddress('10.10.10.1')

        routes = (
            StaticRoute(name='Test Route 1', vrf=vrf, prefix='0.0.0.0/0', next_hop=nh),
            StaticRoute(
                name='Test Route 2', vrf=None, prefix='1.1.1.1/32', next_hop=nh
            ),
            StaticRoute(name='Test Route 3', vrf=vrf, prefix='2.2.2.2/32', next_hop=nh),
        )
        StaticRoute.objects.bulk_create(routes)

        routes[0].devices.set([device])
        routes[1].devices.set([device])
        routes[2].devices.set([device])

        cls.create_data = [
            {
                'name': 'Default Route',
                'devices': [device.pk],
                'vrf': vrf.pk,
                'prefix': '0.0.0.0/0',
                'next_hop': '10.10.10.2',
                'metric': 1,
                'permanent': True,
            },
            {
                'name': 'Google DNS',
                'devices': [device.pk],
                'vrf': None,
                'prefix': '4.4.4.4/32',
                'next_hop': '10.10.10.1',
                'metric': 1,
                'permanent': True,
            },
            {
                'name': 'One dot one dot one dot one',
                'devices': [device.pk],
                'vrf': None,
                'prefix': '1.1.1.0/24',
                'next_hop': '10.10.10.1',
                'metric': 1,
                'permanent': True,
            },
        ]



from django.utils.translation import gettext as _

from dcim.models import Interface, Device
from ipam.models import Prefix, VRF
from netbox.forms import NetBoxModelImportForm
from utilities.forms.fields import CSVModelChoiceField

from netbox_routing.models import *

__all__ = (
    'EIGRPRouterImportForm',
    'EIGRPAddressFamilyImportForm',
    'EIGRPNetworkImportForm',
    'EIGRPInterfaceImportForm',
)


class EIGRPRouterImportForm(NetBoxModelImportForm):
    device = CSVModelChoiceField(
        queryset=Device.objects.all(),
        required=True,
        to_field_name='name',
        help_text=_('Name of device'),
    )

    class Meta:
        model = EIGRPRouter
        fields = (
            'device',
            'rid',
            'mode',
            'name',
            'pid',
            'description',
            'comments',
            'tags',
        )


class EIGRPAddressFamilyImportForm(NetBoxModelImportForm):
    router = CSVModelChoiceField(
        queryset=EIGRPRouter.objects.all(),
        required=True,
        to_field_name='name',
        help_text=_('Name of device'),
    )
    vrf = CSVModelChoiceField(
        queryset=VRF.objects.all(),
        required=False,
        to_field_name='name',
        help_text=_('Name of VRF (if applicable)'),
    )

    class Meta:
        model = EIGRPAddressFamily
        fields = (
            'router',
            'family',
            'vrf',
            'rid',
            'description',
            'comments',
            'tags',
        )


class EIGRPNetworkImportForm(NetBoxModelImportForm):
    router = CSVModelChoiceField(
        queryset=EIGRPRouter.objects.all(),
        required=True,
        to_field_name='name',
        help_text=_('PK of Router Instance'),
    )
    address_family = CSVModelChoiceField(
        queryset=EIGRPAddressFamily.objects.all(),
        required=False,
        help_text=_('PK of Address Family'),
    )
    network = CSVModelChoiceField(
        queryset=Prefix.objects.all(),
        required=True,
        to_field_name='prefix',
        help_text=_('Prefix of Network'),
    )

    class Meta:
        model = EIGRPNetwork
        fields = (
            'router',
            'address_family',
            'network',
            'description',
            'comments',
            'tags',
        )


class EIGRPInterfaceImportForm(NetBoxModelImportForm):
    device = CSVModelChoiceField(
        queryset=Device.objects.all(),
        required=False,
        to_field_name='name',
        help_text=_('Name of device'),
    )
    router = CSVModelChoiceField(
        queryset=EIGRPRouter.objects.all(),
        required=True,
        help_text=_('PK of Router Instance'),
    )
    address_family = CSVModelChoiceField(
        queryset=EIGRPAddressFamily.objects.all(),
        required=False,
        help_text=_('PK of Address Family'),
    )
    interface = CSVModelChoiceField(
        queryset=Interface.objects.all(),
        required=True,
        to_field_name='name',
        help_text=_('Name of interface'),
    )

    class Meta:
        model = EIGRPInterface
        fields = (
            'device',
            'router',
            'address_family',
            'interface',
            'passive',
            'bfd',
            'authentication',
            'passphrase',
            'description',
            'comments',
            'tags',
        )



import django_filters
from django.core.exceptions import ValidationError
from django.db.models import Q
from django.utils.translation import gettext as _

from dcim.models import Device, Interface
from ipam.models import VRF
from utilities.filters import MultiValueCharFilter
from utilities.filtersets import register_filterset

from netbox.filtersets import NetBoxModelFilterSet
from netbox_routing.choices.ospf import OSPFAreaTypeChoices
from netbox_routing.models import OSPFArea, OSPFInstance, OSPFInterface

__all__ = ('OSPFAreaFilterSet', 'OSPFInstanceFilterSet', 'OSPFInterfaceFilterSet')


@register_filterset
class OSPFInstanceFilterSet(NetBoxModelFilterSet):
    device_id = django_filters.ModelMultipleChoiceFilter(
        field_name='device',
        queryset=Device.objects.all(),
        label='Device (ID)',
    )
    device = django_filters.ModelMultipleChoiceFilter(
        field_name='device__name',
        queryset=Device.objects.all(),
        to_field_name='name',
        label='Device',
    )
    vrf_id = django_filters.ModelMultipleChoiceFilter(
        field_name='vrf',
        queryset=VRF.objects.all(),
        label='VRF (ID)',
    )
    vrf = django_filters.ModelMultipleChoiceFilter(
        field_name='vrf__name',
        queryset=VRF.objects.all(),
        to_field_name='name',
        label='VRF',
    )

    router_id = MultiValueCharFilter(
        method='filter_rid',
        label=_('Router ID'),
    )

    class Meta:
        model = OSPFInstance
        fields = (
            'device_id',
            'device',
            'name',
            'vrf_id',
            'vrf',
            'router_id',
            'process_id',
        )

    def search(self, queryset, name, value):
        if not value.strip():
            return queryset
        qs_filter = (
            Q(name__icontains=value)
            | Q(device__name__icontains=value)
            | Q(router_id__icontains=value)
        )
        return queryset.filter(qs_filter).distinct()

    def filter_rid(self, queryset, name, value):
        try:
            return queryset.filter(**{f'{name}__in': value})
        except ValidationError:
            return queryset.none()


@register_filterset
class OSPFAreaFilterSet(NetBoxModelFilterSet):
    device_id = django_filters.ModelMultipleChoiceFilter(
        field_name='interfaces__interface__device__id',
        queryset=Device.objects.all(),
        to_field_name='id',
        label=_('Device (ID)'),
    )

    area_id = MultiValueCharFilter(
        method='filter_aid',
        label=_('Area ID'),
    )

    area_type = django_filters.MultipleChoiceFilter(
        choices=OSPFAreaTypeChoices,
        label=_('Area Type'),
    )

    class Meta:
        model = OSPFArea
        fields = ('area_id', 'area_type')

    def search(self, queryset, name, value):
        if not value.strip():
            return queryset
        qs_filter = Q(area_id__icontains=value)
        return queryset.filter(qs_filter).distinct()

    def filter_aid(self, queryset, name, value):
        try:
            return queryset.filter(**{f'{name}__in': value})
        except ValidationError:
            return queryset.none()


@register_filterset
class OSPFInterfaceFilterSet(NetBoxModelFilterSet):
    instance_id = django_filters.ModelMultipleChoiceFilter(
        field_name='instance',
        queryset=OSPFInstance.objects.all(),
        label='Instance (ID)',
    )
    instance = django_filters.ModelMultipleChoiceFilter(
        field_name='instance__name',
        queryset=OSPFInstance.objects.all(),
        to_field_name='name',
        label='Instance',
    )
    vrf_id = django_filters.ModelMultipleChoiceFilter(
        field_name='instance__vrf',
        queryset=VRF.objects.all(),
        label='VRF (ID)',
    )
    vrf = django_filters.ModelMultipleChoiceFilter(
        field_name='instance__vrf__name',
        queryset=VRF.objects.all(),
        to_field_name='name',
        label='VRF',
    )
    area_id = django_filters.ModelMultipleChoiceFilter(
        field_name='area',
        queryset=OSPFArea.objects.all(),
        label='Area (ID)',
    )
    area = django_filters.ModelMultipleChoiceFilter(
        field_name='area__area_id',
        queryset=OSPFArea.objects.all(),
        to_field_name='area_id',
        label='Area',
    )
    device_id = django_filters.ModelMultipleChoiceFilter(
        field_name='interface__device',
        queryset=Device.objects.all(),
        label='Device (ID)',
    )
    device = django_filters.ModelMultipleChoiceFilter(
        field_name='interface__device__name',
        queryset=Device.objects.all(),
        to_field_name='name',
        label='Device',
    )
    interface_id = django_filters.ModelMultipleChoiceFilter(
        field_name='interface',
        queryset=Interface.objects.all(),
        label='Area (ID)',
    )
    interface = django_filters.ModelMultipleChoiceFilter(
        field_name='interface__name',
        queryset=Interface.objects.all(),
        to_field_name='name',
        label='Area',
    )

    class Meta:
        model = OSPFInterface
        fields = (
            'instance',
            'area',
            'interface',
            'passive',
            'bfd',
            'priority',
            'authentication',
            'passphrase',
        )

    def search(self, queryset, name, value):
        if not value.strip():
            return queryset
        qs_filter = (
            Q(instance__name__icontains=value)
            | Q(area__area_id__icontains=value)
            | Q(interface__name__icontains=value)
            | Q(interface__label__icontains=value)
            | Q(interface__device__name__icontains=value)
        )
        return queryset.filter(qs_filter).distinct()



from django import forms
from django.core.exceptions import ValidationError
from django.utils.translation import gettext as _

from dcim.models import Interface, Device
from ipam.models import VRF
from netbox.forms import NetBoxModelForm
from utilities.forms import BOOLEAN_WITH_BLANK_CHOICES
from utilities.forms.fields import DynamicModelChoiceField, CommentField
from utilities.forms.rendering import FieldSet

from netbox_routing.models import OSPFArea, OSPFInstance, OSPFInterface

__all__ = (
    'OSPFAreaForm',
    'OSPFInstanceForm',
    'OSPFInterfaceForm',
)


class OSPFInstanceForm(NetBoxModelForm):
    device = DynamicModelChoiceField(
        queryset=Device.objects.all(),
        required=True,
        selector=True,
        label=_('Device'),
    )
    vrf = DynamicModelChoiceField(
        queryset=VRF.objects.all(),
        required=False,
        selector=True,
        label=_('VRF'),
    )
    comments = CommentField()

    fieldsets = (
        FieldSet(
            'name',
            'description',
        ),
        FieldSet(
            'device',
            name=_('Device'),
        ),
        FieldSet(
            'process_id',
            'router_id',
            'vrf',
            name=_('Instance'),
        ),
    )

    class Meta:
        model = OSPFInstance
        fields = (
            'name',
            'router_id',
            'process_id',
            'device',
            'vrf',
            'description',
            'comments',
        )


class OSPFAreaForm(NetBoxModelForm):
    comments = CommentField()

    class Meta:
        model = OSPFArea
        fields = (
            'area_id',
            'area_type',
            'description',
            'comments',
        )


class OSPFInterfaceForm(NetBoxModelForm):
    device = DynamicModelChoiceField(
        queryset=Device.objects.all(),
        required=False,
        selector=True,
        label=_('Device'),
    )
    instance = DynamicModelChoiceField(
        queryset=OSPFInstance.objects.all(),
        required=True,
        selector=True,
        label=_('Instance'),
        query_params={
            'device_id': '$device',
        },
    )
    area = DynamicModelChoiceField(
        queryset=OSPFArea.objects.all(),
        required=True,
        selector=True,
        label=_('Area'),
    )
    interface = DynamicModelChoiceField(
        queryset=Interface.objects.all(),
        required=True,
        selector=True,
        label=_('Interface'),
        query_params={
            'device_id': '$device',
        },
    )
    comments = CommentField()

    fieldsets = (
        FieldSet(
            'area',
            'description',
            name=_('Session'),
        ),
        FieldSet(
            'device',
            'instance',
            name=_('Instance'),
        ),
        FieldSet(
            'interface',
            'priority',
            'passive',
            'bfd',
            name=_('Interface'),
        ),
        FieldSet(
            'authentication',
            'passphrase',
            name=_('Authentication'),
        ),
    )

    class Meta:
        model = OSPFInterface
        fields = (
            'device',
            'instance',
            'area',
            'interface',
            'passive',
            'priority',
            'bfd',
            'authentication',
            'passphrase',
            'description',
            'comments',
        )

        widgets = {
            'bfd': forms.Select(choices=BOOLEAN_WITH_BLANK_CHOICES),
            'passive': forms.Select(choices=BOOLEAN_WITH_BLANK_CHOICES),
        }

    def __init__(self, *args, **kwargs):
        super().__init__(*args, **kwargs)
        if self.instance.pk:
            self.initial['device'] = self.instance.interface.device.pk

    def clean(self):
        super().clean()
        if self.cleaned_data.get('instance') and self.cleaned_data.get('interface'):
            if (
                self.cleaned_data.get('instance').device
                != self.cleaned_data.get('interface').device
            ):
                raise ValidationError(
                    {
                        'instance': _(
                            'OSPF Instance Device and Interface Device must match'
                        ),
                        'interface': _(
                            'OSPF Instance Device and Interface Device must match'
                        ),
                    }
                )



from netbox.api.viewsets import NetBoxModelViewSet
from netbox_routing import filtersets
from netbox_routing.api._serializers.community import *
from netbox_routing.models.community import *

__all__ = (
    'CommunityListViewSet',
    'CommunityViewSet',
    'CommunityListEntryViewSet',
)


class CommunityListViewSet(NetBoxModelViewSet):
    queryset = CommunityList.objects.all()
    serializer_class = CommunityListSerializer
    filterset_class = filtersets.CommunityListFilterSet


class CommunityViewSet(NetBoxModelViewSet):
    queryset = Community.objects.all()
    serializer_class = CommunitySerializer
    filterset_class = filtersets.CommunityFilterSet


class CommunityListEntryViewSet(NetBoxModelViewSet):
    queryset = CommunityListEntry.objects.all()
    serializer_class = CommunityListEntrySerializer
    filterset_class = filtersets.CommunityListEntryFilterSet



import strawberry_django
from strawberry_django import FilterLookup

from netbox.graphql.filters import PrimaryModelFilter
from netbox_routing import models
from netbox_routing.graphql.filter_mixins import VRFMixin, DeviceMixin

__all__ = ('StaticRouteFilter',)


@strawberry_django.filter(models.StaticRoute, lookups=True)
class StaticRouteFilter(VRFMixin, DeviceMixin, PrimaryModelFilter):
    prefix: FilterLookup[str] | None = strawberry_django.filter_field()
    next_hop: FilterLookup[str] | None = strawberry_django.filter_field()



# Generated by Django 5.0.8 on 2024-09-14 01:04

from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        (
            'netbox_routing',
            '0007_bgpsessiontemplate_asn_bgpsessiontemplate_bfd_and_more',
        ),
    ]

    operations = [
        migrations.AddField(
            model_name='bgpaddressfamily',
            name='comments',
            field=models.TextField(blank=True),
        ),
        migrations.AddField(
            model_name='bgpaddressfamily',
            name='description',
            field=models.CharField(blank=True, max_length=200),
        ),
        migrations.AddField(
            model_name='bgppeer',
            name='comments',
            field=models.TextField(blank=True),
        ),
        migrations.AddField(
            model_name='bgppeer',
            name='description',
            field=models.CharField(blank=True, max_length=200),
        ),
        migrations.AddField(
            model_name='bgppeeraddressfamily',
            name='comments',
            field=models.TextField(blank=True),
        ),
        migrations.AddField(
            model_name='bgppeeraddressfamily',
            name='description',
            field=models.CharField(blank=True, max_length=200),
        ),
        migrations.AddField(
            model_name='bgppeertemplate',
            name='comments',
            field=models.TextField(blank=True),
        ),
        migrations.AddField(
            model_name='bgppeertemplate',
            name='description',
            field=models.CharField(blank=True, max_length=200),
        ),
        migrations.AddField(
            model_name='bgppoliytemplate',
            name='comments',
            field=models.TextField(blank=True),
        ),
        migrations.AddField(
            model_name='bgppoliytemplate',
            name='description',
            field=models.CharField(blank=True, max_length=200),
        ),
        migrations.AddField(
            model_name='bgprouter',
            name='comments',
            field=models.TextField(blank=True),
        ),
        migrations.AddField(
            model_name='bgprouter',
            name='description',
            field=models.CharField(blank=True, max_length=200),
        ),
        migrations.AddField(
            model_name='bgpscope',
            name='comments',
            field=models.TextField(blank=True),
        ),
        migrations.AddField(
            model_name='bgpscope',
            name='description',
            field=models.CharField(blank=True, max_length=200),
        ),
        migrations.AddField(
            model_name='bgpsessiontemplate',
            name='comments',
            field=models.TextField(blank=True),
        ),
        migrations.AddField(
            model_name='bgpsessiontemplate',
            name='description',
            field=models.CharField(blank=True, max_length=200),
        ),
        migrations.AddField(
            model_name='bgpsetting',
            name='comments',
            field=models.TextField(blank=True),
        ),
        migrations.AddField(
            model_name='bgpsetting',
            name='description',
            field=models.CharField(blank=True, max_length=200),
        ),
        migrations.AddField(
            model_name='ospfarea',
            name='comments',
            field=models.TextField(blank=True),
        ),
        migrations.AddField(
            model_name='ospfarea',
            name='description',
            field=models.CharField(blank=True, max_length=200),
        ),
        migrations.AddField(
            model_name='ospfinstance',
            name='comments',
            field=models.TextField(blank=True),
        ),
        migrations.AddField(
            model_name='ospfinstance',
            name='description',
            field=models.CharField(blank=True, max_length=200),
        ),
        migrations.AddField(
            model_name='ospfinterface',
            name='comments',
            field=models.TextField(blank=True),
        ),
        migrations.AddField(
            model_name='ospfinterface',
            name='description',
            field=models.CharField(blank=True, max_length=200),
        ),
        migrations.AddField(
            model_name='prefixlist',
            name='comments',
            field=models.TextField(blank=True),
        ),
        migrations.AddField(
            model_name='prefixlist',
            name='description',
            field=models.CharField(blank=True, max_length=200),
        ),
        migrations.AddField(
            model_name='prefixlistentry',
            name='comments',
            field=models.TextField(blank=True),
        ),
        migrations.AddField(
            model_name='prefixlistentry',
            name='description',
            field=models.CharField(blank=True, max_length=200),
        ),
        migrations.AddField(
            model_name='routemap',
            name='comments',
            field=models.TextField(blank=True),
        ),
        migrations.AddField(
            model_name='routemap',
            name='description',
            field=models.CharField(blank=True, max_length=200),
        ),
        migrations.AddField(
            model_name='routemapentry',
            name='comments',
            field=models.TextField(blank=True),
        ),
        migrations.AddField(
            model_name='routemapentry',
            name='description',
            field=models.CharField(blank=True, max_length=200),
        ),
        migrations.AddField(
            model_name='staticroute',
            name='comments',
            field=models.TextField(blank=True),
        ),
        migrations.AddField(
            model_name='staticroute',
            name='description',
            field=models.CharField(blank=True, max_length=200),
        ),
    ]



# Generated by Django 5.2.5 on 2026-01-16 23:42

import django.core.validators
import django.db.models.deletion
import netbox.models.deletion
import taggit.managers
import utilities.json
from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        ('netbox_routing', '0019_prefixlist_routemap_changes'),
    ]

    operations = [
        migrations.CreateModel(
            name='Community',
            fields=[
                (
                    'id',
                    models.BigAutoField(
                        auto_created=True, primary_key=True, serialize=False
                    ),
                ),
                ('created', models.DateTimeField(auto_now_add=True, null=True)),
                ('last_updated', models.DateTimeField(auto_now=True, null=True)),
                (
                    'custom_field_data',
                    models.JSONField(
                        blank=True,
                        default=dict,
                        encoder=utilities.json.CustomFieldJSONEncoder,
                    ),
                ),
                ('description', models.CharField(blank=True, max_length=200)),
                ('comments', models.TextField(blank=True)),
                (
                    'community',
                    models.CharField(
                        max_length=255,
                        validators=[
                            django.core.validators.RegexValidator(
                                '^[\\d\\.]+(?::[\\d\\.]+(?::[\\d\\.]+)?)?$'
                            )
                        ],
                    ),
                ),
                ('status', models.CharField(default='active', max_length=50)),
                (
                    'owner',
                    models.ForeignKey(
                        blank=True,
                        null=True,
                        on_delete=django.db.models.deletion.PROTECT,
                        to='users.owner',
                    ),
                ),
                (
                    'role',
                    models.ForeignKey(
                        blank=True,
                        null=True,
                        on_delete=django.db.models.deletion.SET_NULL,
                        related_name='communities',
                        to='ipam.role',
                    ),
                ),
                (
                    'tags',
                    taggit.managers.TaggableManager(
                        through='extras.TaggedItem', to='extras.Tag'
                    ),
                ),
                (
                    'tenant',
                    models.ForeignKey(
                        blank=True,
                        null=True,
                        on_delete=django.db.models.deletion.PROTECT,
                        related_name='communities',
                        to='tenancy.tenant',
                    ),
                ),
            ],
            options={
                'verbose_name': 'Community',
                'verbose_name_plural': 'Communities',
                'ordering': ['community'],
            },
            bases=(netbox.models.deletion.DeleteMixin, models.Model),
        ),
        migrations.CreateModel(
            name='CommunityList',
            fields=[
                (
                    'id',
                    models.BigAutoField(
                        auto_created=True, primary_key=True, serialize=False
                    ),
                ),
                ('created', models.DateTimeField(auto_now_add=True, null=True)),
                ('last_updated', models.DateTimeField(auto_now=True, null=True)),
                (
                    'custom_field_data',
                    models.JSONField(
                        blank=True,
                        default=dict,
                        encoder=utilities.json.CustomFieldJSONEncoder,
                    ),
                ),
                ('description', models.CharField(blank=True, max_length=200)),
                ('comments', models.TextField(blank=True)),
                ('name', models.CharField(max_length=100)),
                (
                    'owner',
                    models.ForeignKey(
                        blank=True,
                        null=True,
                        on_delete=django.db.models.deletion.PROTECT,
                        to='users.owner',
                    ),
                ),
                (
                    'tags',
                    taggit.managers.TaggableManager(
                        through='extras.TaggedItem', to='extras.Tag'
                    ),
                ),
                (
                    'tenant',
                    models.ForeignKey(
                        blank=True,
                        null=True,
                        on_delete=django.db.models.deletion.PROTECT,
                        related_name='community_lists',
                        to='tenancy.tenant',
                    ),
                ),
            ],
            options={
                'verbose_name': 'Community List',
                'verbose_name_plural': 'Community Lists',
                'ordering': ['name'],
                'unique_together': {('name',)},
            },
            bases=(netbox.models.deletion.DeleteMixin, models.Model),
        ),
        migrations.CreateModel(
            name='CommunityListEntry',
            fields=[
                (
                    'id',
                    models.BigAutoField(
                        auto_created=True, primary_key=True, serialize=False
                    ),
                ),
                ('created', models.DateTimeField(auto_now_add=True, null=True)),
                ('last_updated', models.DateTimeField(auto_now=True, null=True)),
                (
                    'custom_field_data',
                    models.JSONField(
                        blank=True,
                        default=dict,
                        encoder=utilities.json.CustomFieldJSONEncoder,
                    ),
                ),
                ('description', models.CharField(blank=True, max_length=200)),
                ('comments', models.TextField(blank=True)),
                ('action', models.CharField(max_length=30)),
                (
                    'community',
                    models.ForeignKey(
                        on_delete=django.db.models.deletion.CASCADE,
                        related_name='communitylistentries',
                        to='netbox_routing.community',
                    ),
                ),
                (
                    'community_list',
                    models.ForeignKey(
                        on_delete=django.db.models.deletion.CASCADE,
                        related_name='communitylistentries',
                        to='netbox_routing.communitylist',
                    ),
                ),
                (
                    'owner',
                    models.ForeignKey(
                        blank=True,
                        null=True,
                        on_delete=django.db.models.deletion.PROTECT,
                        to='users.owner',
                    ),
                ),
                (
                    'tags',
                    taggit.managers.TaggableManager(
                        through='extras.TaggedItem', to='extras.Tag'
                    ),
                ),
            ],
            options={
                'ordering': ('community_list', 'community'),
            },
            bases=(netbox.models.deletion.DeleteMixin, models.Model),
        ),
        migrations.AddConstraint(
            model_name='community',
            constraint=models.UniqueConstraint(
                fields=('community',), name='netbox_routing_community_community'
            ),
        ),
    ]



# Generated by Django 4.1.7 on 2023-05-18 13:43

from django.db import migrations, models
import utilities.json


class Migration(migrations.Migration):

    dependencies = [
        ('netbox_routing', '0001_initial'),
    ]

    operations = [
        migrations.AlterField(
            model_name='prefixlist',
            name='custom_field_data',
            field=models.JSONField(
                blank=True, default=dict, encoder=utilities.json.CustomFieldJSONEncoder
            ),
        ),
        migrations.AlterField(
            model_name='prefixlistentry',
            name='custom_field_data',
            field=models.JSONField(
                blank=True, default=dict, encoder=utilities.json.CustomFieldJSONEncoder
            ),
        ),
        migrations.AlterField(
            model_name='routemap',
            name='custom_field_data',
            field=models.JSONField(
                blank=True, default=dict, encoder=utilities.json.CustomFieldJSONEncoder
            ),
        ),
        migrations.AlterField(
            model_name='routemapentry',
            name='custom_field_data',
            field=models.JSONField(
                blank=True, default=dict, encoder=utilities.json.CustomFieldJSONEncoder
            ),
        ),
        migrations.AlterField(
            model_name='staticroute',
            name='custom_field_data',
            field=models.JSONField(
                blank=True, default=dict, encoder=utilities.json.CustomFieldJSONEncoder
            ),
        ),
    ]



from utilities.testing import APIViewTestCases

from netbox_routing.models.bgp import *
from netbox_routing.tests.base import *
from netbox_routing.tests.bgp.base import *

__all__ = (
    'BGPPeerTemplateTestCase',
    'BGPPolicyTemplateTestCase',
    'BGPSessionTemplateTestCase',
)


class BGPPeerTemplateTestCase(APIViewTestCases.APIViewTestCase):
    model = BGPPeerTemplate
    view_namespace = "plugins-api:netbox_routing"
    brief_fields = [
        'display',
        'id',
        'name',
        'remote_as',
        'url',
    ]

    bulk_update_data = {'description': 'Description'}
    routing_required_fields = ('name',)

    @classmethod
    def setUpTestData(cls):
        cls.instances = (
            cls.model(
                name=f'{cls.model._meta.verbose_name} {i}',
            )
            for i in range(1, 5)
        )
        cls.model.objects.bulk_create(cls.instances)
        cls.create_data = [
            {'name': f'{cls.model._meta.verbose_name} {i}'} for i in range(5, 7)
        ]


class BGPPolicyTemplateTestCase(APIViewTestCases.APIViewTestCase):
    model = BGPPolicyTemplate
    view_namespace = "plugins-api:netbox_routing"
    brief_fields = [
        'display',
        'id',
        'name',
        'url',
    ]

    bulk_update_data = {'description': 'Description'}
    routing_required_fields = ('name',)

    @classmethod
    def setUpTestData(cls):
        cls.instances = (
            cls.model(
                name=f'{cls.model._meta.verbose_name} {i}',
            )
            for i in range(1, 5)
        )
        cls.model.objects.bulk_create(cls.instances)

        cls.create_data = [
            {'name': f'{cls.model._meta.verbose_name} {i}'} for i in range(5, 7)
        ]


class BGPSessionTemplateTestCase(APIViewTestCases.APIViewTestCase):
    model = BGPSessionTemplate
    view_namespace = "plugins-api:netbox_routing"
    brief_fields = [
        'display',
        'id',
        'name',
        'parent',
        'url',
    ]

    bulk_update_data = {'description': 'Description'}
    routing_required_fields = ('name',)

    @classmethod
    def setUpTestData(cls):
        cls.instances = (
            cls.model(
                name=f'{cls.model._meta.verbose_name} {i}',
            )
            for i in range(1, 5)
        )
        cls.model.objects.bulk_create(cls.instances)

        cls.create_data = [
            {'name': f'{cls.model._meta.verbose_name} {i}'} for i in range(5, 7)
        ]


class BGPRouterTestCase(
    AutomatedModelCreationMixin, ASNMixin, APIViewTestCases.APIViewTestCase
):
    model = BGPRouter
    view_namespace = "plugins-api:netbox_routing"
    brief_fields = [
        'asn',
        'assigned_object',
        'display',
        'id',
        'name',
        'url',
    ]

    bulk_update_data = {'description': 'Description'}
    routing_required_fields = (
        'name',
        ('asn', 'asns'),
    )

    @classmethod
    def setUpTestData(cls):
        super().setUpTestData()

        cls.create_data = [
            {'name': f'{cls.model._meta.verbose_name}: {i}', 'asn': cls.asns[i].pk}
            for i in range(5, 7)
        ]


class BGPScopeTestCase(
    AutomatedModelCreationMixin,
    BGPRouterMixin,
    VRFMixin,
    APIViewTestCases.APIViewTestCase,
):
    model = BGPScope
    view_namespace = "plugins-api:netbox_routing"
    brief_fields = [
        'display',
        'id',
        'router',
        'url',
        'vrf',
    ]

    bulk_update_data = {'description': 'Description'}
    routing_required_fields = (
        'router',
        ('vrf', 'vrfs'),
    )

    @classmethod
    def setUpTestData(cls):
        super().setUpTestData()

        cls.create_data = [
            {'router': cls.router.pk, 'vrf': cls.vrfs[i].pk} for i in range(5, 7)
        ]


class BGPAddressFamilyTestCase(
    AutomatedModelCreationMixin, BGPScopeMixin, APIViewTestCases.APIViewTestCase
):
    model = BGPAddressFamily
    view_namespace = "plugins-api:netbox_routing"
    brief_fields = [
        'address_family',
        'display',
        'id',
        'scope',
        'url',
    ]

    bulk_update_data = {'description': 'Description'}
    routing_required_fields = (
        'scope',
        ('address_family', 'afs'),
    )
    afs = (
        'ipv4-unicast',
        'ipv6-unicast',
        'vpnv4-unicast',
        'vpnv6-unicast',
        'l2vpn-vpls',
        'l2vpn-evpn',
        'ipv4-multicast',
        'ipv6-multicast',
    )

    @classmethod
    def setUpTestData(cls):
        super().setUpTestData()

        cls.create_data = [
            {'scope': cls.scope.pk, 'address_family': cls.afs[i]} for i in range(5, 7)
        ]


class BGPPeerTestCase(
    AutomatedModelCreationMixin,
    AddressesMixin,
    BGPScopeMixin,
    APIViewTestCases.APIViewTestCase,
):
    model = BGPPeer
    view_namespace = "plugins-api:netbox_routing"
    brief_fields = [
        'display',
        'enabled',
        'id',
        'name',
        'peer',
        'remote_as',
        'scope',
        'status',
        'url',
    ]

    bulk_update_data = {'description': 'Description'}
    enabled = True
    routing_required_fields = (
        'name',
        'scope',
        'enabled',
        ('remote_as', 'asn'),
        ('peer', 'ip_addresses'),
    )

    @classmethod
    def setUpTestData(cls):
        super().setUpTestData()

        cls.create_data = [
            {
                'name': f'{cls.model._meta.verbose_name}: {i}',
                'scope': cls.scope.pk,
                'enabled': cls.enabled,
                'remote_as': cls.asn.pk,
                'peer': cls.ip_addresses[i].pk,
            }
            for i in range(5, 7)
        ]


class BGPPeerAddressFamilyTestCase(
    AutomatedModelCreationMixin,
    BGPPeerMixin,
    BGPAddressFamilyMixin,
    APIViewTestCases.APIViewTestCase,
):
    model = BGPPeerAddressFamily
    view_namespace = "plugins-api:netbox_routing"
    brief_fields = [
        'address_family',
        'assigned_object',
        'display',
        'enabled',
        'id',
        'url',
    ]

    bulk_update_data = {'description': 'Description'}
    enabled = True
    routing_required_fields = (
        ('assigned_object', 'peer'),
        ('address_family', 'address_families'),
        'enabled',
    )

    @classmethod
    def setUpTestData(cls):
        super().setUpTestData()

        ctype = f'{cls.peer._meta.app_label}.{cls.peer._meta.model_name}'
        cls.create_data = [
            {
                'assigned_object_type': ctype,
                'assigned_object_id': cls.peer.pk,
                'enabled': cls.enabled,
                'address_family': cls.address_families[i].pk,
            }
            for i in range(5, 7)
        ]


class BFDProfileTestCase(
    APIViewTestCases.APIViewTestCase,
):
    model = BFDProfile
    view_namespace = "plugins-api:netbox_routing"
    brief_fields = [
        'display',
        'id',
        'name',
        'url',
    ]

    bulk_update_data = {'description': 'Description'}

    @classmethod
    def setUpTestData(cls):
        super().setUpTestData()
        profiles = []
        for i in range(1, 4):
            profiles.append(
                BFDProfile(
                    name=f'Test BFD Profile {i}',
                    min_rx_int=60,
                    min_tx_int=60,
                    multiplier=3,
                )
            )
        BFDProfile.objects.bulk_create(profiles)

        cls.create_data = [
            {
                'name': f'Test BFD Profile {i}',
                'min_rx_int': 60,
                'min_tx_int': 60,
                'multiplier': 3,
            }
            for i in range(4, 6)
        ]



import strawberry_django

from netbox.graphql.filters import PrimaryModelFilter
from netbox_routing import models

__all__ = (
    'BGPSettingFilter',
    'BGPPeerTemplateFilter',
    'BGPPolicyTemplateFilter',
    'BGPSessionTemplateFilter',
    'BGPRouterFilter',
    'BGPScopeFilter',
    'BGPAddressFamilyFilter',
    'BGPPeerFilter',
    'BGPPeerAddressFamilyFilter',
    'BFDProfileFilter',
)


@strawberry_django.filter(models.BGPPeerTemplate, lookups=True)
class BGPPeerTemplateFilter(PrimaryModelFilter):
    pass


@strawberry_django.filter(models.BGPPolicyTemplate, lookups=True)
class BGPPolicyTemplateFilter(PrimaryModelFilter):
    pass


@strawberry_django.filter(models.BGPSessionTemplate, lookups=True)
class BGPSessionTemplateFilter(PrimaryModelFilter):
    pass


@strawberry_django.filter(models.BGPRouter, lookups=True)
class BGPRouterFilter(PrimaryModelFilter):
    pass


@strawberry_django.filter(models.BGPScope, lookups=True)
class BGPScopeFilter(PrimaryModelFilter):
    pass


@strawberry_django.filter(models.BGPAddressFamily, lookups=True)
class BGPAddressFamilyFilter(PrimaryModelFilter):
    pass


@strawberry_django.filter(models.BGPPeer, lookups=True)
class BGPPeerFilter(PrimaryModelFilter):
    pass


@strawberry_django.filter(models.BGPPeerAddressFamily, lookups=True)
class BGPPeerAddressFamilyFilter(PrimaryModelFilter):
    pass


@strawberry_django.filter(models.BGPSetting, lookups=True)
class BGPSettingFilter(PrimaryModelFilter):
    pass


@strawberry_django.filter(models.BFDProfile, lookups=True)
class BFDProfileFilter(PrimaryModelFilter):
    pass



# from django.core.exceptions import ValidationError
from django.core.exceptions import ValidationError
from django.db import IntegrityError
from django.test import TestCase

from ipam.models import Prefix
from netbox_routing.models.objects import *

__all__ = (
    'ASPathTestCase',
    'ASPathEntryTestCase',
    'PrefixListTestCase',
    'PrefixListEntryTestCase',
    'RouteMapTestCase',
    'RouteMapEntryTestCase',
)


class ASPathTestCase(TestCase):

    @classmethod
    def setUpTestData(cls):
        pass

    def test_community(self):
        name = 'AS Path 1'
        asp = ASPath(
            name=name,
        )
        asp.full_clean()
        asp.save()
        self.assertIsInstance(asp, ASPath)
        self.assertEqual(asp.__str__(), name)

    def test_unique_together(self):

        name = 'AS Path 1'
        asp = ASPath(
            name=name,
        )
        asp.full_clean()
        asp.save()

        asp = ASPath(
            name=name,
        )

        with self.assertRaises(ValidationError):
            asp.full_clean()
        with self.assertRaises(IntegrityError):
            asp.save()


class ASPathEntryTestCase(TestCase):

    @classmethod
    def setUpTestData(cls):
        cls.aspath = ASPath(
            name='AS Path',
        )
        cls.aspath.full_clean()
        cls.aspath.save()

    def test_aspath_entry(self):
        for seq, pattern in (
            (1, '^1$'),
            (2, '^2$'),
            (3, '^3$'),
            (4, '^4$'),
        ):
            aspe = ASPathEntry(
                aspath=self.aspath,
                action='permit',
                sequence=seq,
                pattern=pattern,
            )
            aspe.full_clean()
            aspe.save()
            self.assertIsInstance(aspe, ASPathEntry)
            self.assertEqual(aspe.__str__(), f'{self.aspath} permit {seq}')


class PrefixListTestCase(TestCase):

    @classmethod
    def setUpTestData(cls):
        pass

    def test_community(self):
        name = 'Prefix List 1'
        pl = PrefixList(
            name=name,
        )
        pl.full_clean()
        pl.save()
        self.assertIsInstance(pl, PrefixList)
        self.assertEqual(pl.__str__(), name)

    def test_unique_together(self):

        name = 'Prefix List 1'
        pl = PrefixList(
            name=name,
        )
        pl.full_clean()
        pl.save()

        pl = PrefixList(
            name=name,
        )

        with self.assertRaises(ValidationError):
            pl.full_clean()
        with self.assertRaises(IntegrityError):
            pl.save()


class PrefixListEntryTestCase(TestCase):

    @classmethod
    def setUpTestData(cls):
        cls.prefix_list = PrefixList(
            name='Prefix List',
        )
        cls.prefix_list.full_clean()
        cls.prefix_list.save()

    def test_prefix_list_entry(self):
        for seq, pattern in (
            (1, '10.0.1.0/24'),
            (2, '10.0.2.0/24'),
            (3, '10.0.3.0/24'),
            (4, '10.0.4.0/24'),
        ):
            if seq in [
                1,
                3,
            ]:
                prefix = CustomPrefix(prefix=pattern)
            else:
                prefix = Prefix(prefix=pattern)
            prefix.full_clean()
            prefix.save()

            ple = PrefixListEntry(
                prefix_list=self.prefix_list,
                action='permit',
                sequence=seq,
                assigned_prefix=prefix,
            )
            ple.full_clean()
            ple.save()
            self.assertIsInstance(ple, PrefixListEntry)
            self.assertEqual(ple.__str__(), f'{self.prefix_list} permit {seq}')


class RouteMapTestCase(TestCase):

    @classmethod
    def setUpTestData(cls):
        pass

    def test_community(self):
        name = 'Route Map 1'
        rm = RouteMap(
            name=name,
        )
        rm.full_clean()
        rm.save()
        self.assertIsInstance(rm, RouteMap)
        self.assertEqual(rm.__str__(), name)

    def test_unique_together(self):

        name = 'Route Map 1'
        rm = RouteMap(
            name=name,
        )
        rm.full_clean()
        rm.save()

        rm = RouteMap(
            name=name,
        )

        with self.assertRaises(ValidationError):
            rm.full_clean()
        with self.assertRaises(IntegrityError):
            rm.save()


class RouteMapEntryTestCase(TestCase):

    @classmethod
    def setUpTestData(cls):
        cls.route_map = RouteMap(
            name='Route Map',
        )
        cls.route_map.full_clean()
        cls.route_map.save()

    def test_route_map_entry(self):
        for seq, pattern in (
            (1, '10.0.1.0/24'),
            (2, '10.0.2.0/24'),
            (3, '10.0.3.0/24'),
            (4, '10.0.4.0/24'),
        ):
            rme = RouteMapEntry(
                route_map=self.route_map,
                action='permit',
                sequence=seq,
                match={'tags': f'{seq}{seq}{seq}{seq}'},
            )
            rme.full_clean()
            rme.save()
            self.assertIsInstance(rme, RouteMapEntry)
            self.assertEqual(rme.__str__(), f'{self.route_map} permit {seq}')



import netaddr
from django.utils.translation import gettext_lazy as _
from rest_framework import serializers

__all__ = ('IPAddressField',)


class IPAddressField(serializers.CharField):
    """
    An IPv4 or IPv6 address with optional mask
    """

    default_error_messages = {
        'invalid': _('Enter a valid IPv4 or IPv6 address with optional mask.'),
    }

    def to_internal_value(self, data):
        try:
            return netaddr.IPAddress(data)
        except netaddr.AddrFormatError:
            raise serializers.ValidationError(
                _("Invalid IP address format: {data}").format(data)
            )
        except (TypeError, ValueError) as e:
            raise serializers.ValidationError(e)

    def to_representation(self, value):
        return str(value)



from typing import Annotated

import strawberry
import strawberry_django
from strawberry import ID
from strawberry_django import FilterLookup

from netbox.graphql.filters import PrimaryModelFilter
from netbox_routing import models
from netbox_routing.graphql.filter_mixins import (
    DeviceMixin,
    NetworkPrefixMixin,
    InterfaceMixin,
)

__all__ = (
    'EIGRPRouterFilter',
    'EIGRPAddressFamilyFilter',
    'EIGRPNetworkFilter',
    'EIGRPInterfaceFilter',
)


@strawberry_django.filter(models.EIGRPRouter, lookups=True)
class EIGRPRouterFilter(DeviceMixin, PrimaryModelFilter):
    rid: FilterLookup[str] | None = strawberry_django.filter_field()


@strawberry_django.filter(models.EIGRPAddressFamily, lookups=True)
class EIGRPAddressFamilyFilter(PrimaryModelFilter):
    router: (
        Annotated[
            'EIGRPRouterFilter', strawberry.lazy('netbox_routing.graphql.filters')
        ]
        | None
    ) = strawberry_django.filter_field()
    router_id: ID | None = strawberry_django.filter_field()
    rid: FilterLookup[str] | None = strawberry_django.filter_field()


@strawberry_django.filter(models.EIGRPNetwork, lookups=True)
class EIGRPNetworkFilter(NetworkPrefixMixin, PrimaryModelFilter):
    router: (
        Annotated[
            'EIGRPRouterFilter', strawberry.lazy('netbox_routing.graphql.filters')
        ]
        | None
    ) = strawberry_django.filter_field()
    router_id: ID | None = strawberry_django.filter_field()
    address_family: (
        Annotated[
            'EIGRPAddressFamilyFilter',
            strawberry.lazy('netbox_routing.graphql.filters'),
        ]
        | None
    ) = strawberry_django.filter_field()
    address_family_id: ID | None = strawberry_django.filter_field()


@strawberry_django.filter(models.EIGRPInterface, lookups=True)
class EIGRPInterfaceFilter(InterfaceMixin, PrimaryModelFilter):
    router: (
        Annotated[
            'EIGRPRouterFilter', strawberry.lazy('netbox_routing.graphql.filters')
        ]
        | None
    ) = strawberry_django.filter_field()
    router_id: ID | None = strawberry_django.filter_field()
    address_family: (
        Annotated[
            'EIGRPAddressFamilyFilter',
            strawberry.lazy('netbox_routing.graphql.filters'),
        ]
        | None
    ) = strawberry_django.filter_field()
    address_family_id: ID | None = strawberry_django.filter_field()



from .community import *
from .static import StaticRouteFilterSet
from .objects import *
from .ospf import *
from .bgp import *
from .eigrp import *

__all__ = (
    'StaticRouteFilterSet',
    'BGPSettingFilterSet',
    'BGPRouterFilterSet',
    'BGPScopeFilterSet',
    'BGPAddressFamilyFilterSet',
    'BGPPeerFilterSet',
    'OSPFInstanceFilterSet',
    'OSPFAreaFilterSet',
    'OSPFInterfaceFilterSet',
    'EIGRPRouterFilterSet',
    'EIGRPAddressFamilyFilterSet',
    'EIGRPNetworkFilterSet',
    'EIGRPInterfaceFilterSet',
    'PrefixListFilterSet',
    'PrefixListEntryFilterSet',
    'RouteMapFilterSet',
    'RouteMapEntryFilterSet',
    'CommunityListFilterSet',
    'CommunityListEntryFilterSet',
    'CommunityFilterSet',
    'ASPathFilterSet',
    'ASPathEntryFilterSet',
)



from .community import *
from .static import StaticRouteViewSet
from .ospf import OSPFInstanceViewSet, OSPFAreaViewSet, OSPFInterfaceViewSet
from .bgp import *
from .objects import *
from .eigrp import (
    EIGRPRouterViewSet,
    EIGRPAddressFamilyViewSet,
    EIGRPNetworkViewSet,
    EIGRPInterfaceViewSet,
)

__all__ = (
    'StaticRouteViewSet',
    'BGPSettingViewSet',
    'BGPRouterViewSet',
    'BGPScopeViewSet',
    'BGPAddressFamilyViewSet',
    'BGPPeerViewSet',
    'BGPPeerTemplateViewSet',
    'BGPPolicyTemplateViewSet',
    'BGPSessionTemplateViewSet',
    'BGPPeerAddressFamilyViewSet',
    'BFDProfileViewSet',
    'EIGRPRouterViewSet',
    'EIGRPAddressFamilyViewSet',
    'EIGRPNetworkViewSet',
    'EIGRPInterfaceViewSet',
    'OSPFInstanceViewSet',
    'OSPFAreaViewSet',
    'OSPFInterfaceViewSet',
    'PrefixListViewSet',
    'PrefixListEntryViewSet',
    'RouteMapViewSet',
    'RouteMapEntryViewSet',
    'CommunityViewSet',
    'CommunityListViewSet',
    'CommunityListEntryViewSet',
    'ASPathViewSet',
    'ASPathEntryViewSet',
)



from .ospf import *
from .eigrp import *
from .static import *

__all__ = (
    # Static
    'StaticRouteImportForm',
    # OSPF
    'OSPFInstanceImportForm',
    'OSPFAreaImportForm',
    'OSPFInterfaceImportForm',
    # EIGRP
    'EIGRPRouterImportForm',
    'EIGRPAddressFamilyImportForm',
    'EIGRPNetworkImportForm',
    'EIGRPInterfaceImportForm',
)



# Generated by Django 5.2.9 on 2026-02-17 18:07

import netbox_routing.fields.ip
from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        ('netbox_routing', '0027_bgp_name_optional'),
    ]

    operations = [
        migrations.AddField(
            model_name='staticroute',
            name='interface_next_hop',
            field=models.CharField(blank=True, null=True),
        ),
        migrations.AlterField(
            model_name='staticroute',
            name='next_hop',
            field=netbox_routing.fields.ip.IPAddressField(blank=True, null=True),
        ),
    ]



# Generated by Django 5.0.8 on 2024-09-28 04:42

import django.db.models.deletion
from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        ('netbox_routing', '0011_osfp_passive_interface'),
    ]

    operations = [
        migrations.AddField(
            model_name='ospfinstance',
            name='vrf',
            field=models.ForeignKey(
                blank=True,
                null=True,
                on_delete=django.db.models.deletion.CASCADE,
                related_name='ospf_instances',
                to='ipam.vrf',
            ),
        ),
    ]



from django.contrib.contenttypes.fields import GenericForeignKey, GenericRelation
from django.contrib.contenttypes.models import ContentType
from django.core.validators import MinValueValidator, MaxValueValidator
from django.db import models
from django.db.models import ManyToManyField
from django.urls import reverse
from django.utils.translation import gettext as _

from netbox.models import PrimaryModel
from netbox_routing.choices.bgp import *
from netbox_routing.constants.bgp import *
from netbox_routing.models.base import SearchAttributeMixin

__all__ = (
    'BGPSetting',
    'BGPPeerTemplate',
    'BGPPolicyTemplate',
    'BGPSessionTemplate',
    'BGPRouter',
    'BGPScope',
    'BGPAddressFamily',
    'BGPPeer',
    'BGPPeerAddressFamily',
    'BFDProfile',
)


class BGPSetting(SearchAttributeMixin, PrimaryModel):
    assigned_object_type = models.ForeignKey(
        verbose_name=_('Assigned Object Type'),
        to=ContentType,
        limit_choices_to=BGPSETTING_ASSIGNMENT_MODELS,
        on_delete=models.PROTECT,
        related_name='+',
        blank=True,
        null=True,
    )
    assigned_object_id = models.PositiveBigIntegerField(
        verbose_name=_('Assigned Object ID'), blank=True, null=True
    )
    assigned_object = GenericForeignKey(
        ct_field='assigned_object_type', fk_field='assigned_object_id'
    )
    key = models.CharField(
        verbose_name=_('Setting Name'),
        choices=BGPSettingChoices,
    )
    value = models.CharField(
        verbose_name=_('Setting Value'),
    )

    class Meta:
        verbose_name = 'BGP Setting'
        verbose_name_plural = 'BGP Settings'
        ordering = (
            'assigned_object_type',
            'assigned_object_id',
            'key',
        )
        constraints = [
            models.UniqueConstraint(
                fields=('assigned_object_type', 'assigned_object_id', 'key'),
                name='netbox_routing_bgpsettings_unique',
            ),
        ]

    def __str__(self):
        return f'{self.assigned_object}: {self.key}'

    def get_absolute_url(self):
        return reverse('plugins:netbox_routing:bgpsetting', args=[self.pk])


class BGPSessionTemplate(PrimaryModel):
    name = models.CharField(verbose_name=_('Name'), max_length=100)
    parent = models.ForeignKey(
        verbose_name=_('Parent'),
        to='netbox_routing.BGPSessionTemplate',
        on_delete=models.PROTECT,
        related_name='children',
        blank=True,
        null=True,
    )
    enabled = models.BooleanField(verbose_name=_('Enabled'), blank=True, null=True)
    local_as = models.ForeignKey(
        verbose_name=_('ASN'),
        to='ipam.ASN',
        on_delete=models.PROTECT,
        related_name='session_templates_local_as',
        blank=True,
        null=True,
    )
    remote_as = models.ForeignKey(
        verbose_name=_('ASN'),
        to='ipam.ASN',
        on_delete=models.PROTECT,
        related_name='session_templates_remote_as',
        blank=True,
        null=True,
    )
    ttl = models.PositiveSmallIntegerField(
        verbose_name=_('TTL'),
        blank=True,
        null=True,
        validators=[MinValueValidator(0), MaxValueValidator(255)],
    )
    bfd = models.ForeignKey(
        verbose_name=_('BFD PRofile'),
        to='netbox_routing.BFDProfile',
        on_delete=models.PROTECT,
        related_name='bgp_session_templates',
        blank=True,
        null=True,
    )
    password = models.CharField(
        verbose_name=_('Password'),
        max_length=255,
        blank=True,
        null=True,
    )
    tenant = models.ForeignKey(
        verbose_name=_('Tenant'),
        to='tenancy.Tenant',
        on_delete=models.PROTECT,
        related_name='bgp_peer_session_templates',
        blank=True,
        null=True,
    )

    clone_fields = (
        'name',
        'parent',
        'enabled',
        'remote_as',
        'local_as',
        'bfd',
        'ttl',
        'password',
        'tenant',
    )
    prerequisite_models = ('netbox_routing.BGPRouter',)

    class Meta:
        verbose_name = 'BGP Session Template'
        verbose_name_plural = 'BGP Session Templates'
        ordering = ('name',)
        constraints = [
            models.UniqueConstraint(
                fields=('name',),
                name='%(app_label)s_%(class)s_name',
            ),
        ]

    def __str__(self):
        return f'{self.name}'


class BGPPolicyTemplate(PrimaryModel):
    name = models.CharField(verbose_name=_('Name'), max_length=100)
    parents = models.ManyToManyField(
        verbose_name=_('Parent Policies'),
        to='netbox_routing.BGPPolicyTemplate',
        related_name='children',
    )
    enabled = models.BooleanField(verbose_name=_('Enabled'), blank=True, null=True)
    prefixlist_out = models.ForeignKey(
        verbose_name=_('Outbound Prefix List'),
        to='netbox_routing.PrefixList',
        on_delete=models.PROTECT,
        related_name='template_afs_out',
        blank=True,
        null=True,
    )
    prefixlist_in = models.ForeignKey(
        verbose_name=_('Inbound Prefix List'),
        to='netbox_routing.PrefixList',
        on_delete=models.PROTECT,
        related_name='template_afs_in',
        blank=True,
        null=True,
    )
    routemap_out = models.ForeignKey(
        verbose_name=_('Outbound Route Map'),
        to='netbox_routing.RouteMap',
        on_delete=models.PROTECT,
        related_name='template_afs_out',
        blank=True,
        null=True,
    )
    routemap_in = models.ForeignKey(
        verbose_name=_('Inbound Route Map'),
        to='netbox_routing.RouteMap',
        on_delete=models.PROTECT,
        related_name='template_afs_in',
        blank=True,
        null=True,
    )
    tenant = models.ForeignKey(
        verbose_name=_('Tenant'),
        to='tenancy.Tenant',
        on_delete=models.PROTECT,
        related_name='bgp_peer_policy_templates',
        blank=True,
        null=True,
    )

    clone_fields = (
        'name',
        'parents',
        'enabled',
        'prefixlist_out',
        'prefixlist_in',
        'routemap_out',
        'routemap_in',
    )
    prerequisite_models = ('netbox_routing.BGPRouter',)

    class Meta:
        verbose_name = 'BGP Policy Template'
        verbose_name_plural = 'BGP Policy Templates'
        ordering = ('name',)
        constraints = [
            models.UniqueConstraint(
                fields=('name',),
                name='%(app_label)s_%(class)s_name',
            ),
        ]

    def __str__(self):
        return f'{self.name}'


class BGPPeerTemplate(PrimaryModel):
    name = models.CharField(verbose_name=_('Name'), max_length=100)
    remote_as = models.ForeignKey(
        verbose_name=_('Remote AS'),
        to='ipam.ASN',
        on_delete=models.PROTECT,
        related_name='+',
        blank=True,
        null=True,
    )
    enabled = models.BooleanField(verbose_name=_('Enabled'), blank=True, null=True)
    tenant = models.ForeignKey(
        verbose_name=_('Tenant'),
        to='tenancy.Tenant',
        on_delete=models.PROTECT,
        related_name='bgp_peer_templates',
        blank=True,
        null=True,
    )
    address_families = GenericRelation(
        verbose_name=_('Address Families'),
        to='netbox_routing.BGPPeerAddressFamily',
        related_query_name='peer_group',
        related_name='peer_group',
        content_type_field='assigned_object_type',
        object_id_field='assigned_object_id',
    )

    clone_fields = ('name', 'remote_as', 'enabled')

    class Meta:
        verbose_name = 'BGP Peer Template'
        verbose_name_plural = 'BGP Peer Templates'
        ordering = ('name',)
        constraints = [
            models.UniqueConstraint(
                fields=('name', 'remote_as'),
                name='%(app_label)s_%(class)s_name_remote_as',
            ),
        ]

    def __str__(self):
        return f'{self.name}'


class BGPRouter(SearchAttributeMixin, PrimaryModel):
    name = models.CharField(
        verbose_name=_('Name'), max_length=100, blank=True, null=True
    )
    assigned_object_type = models.ForeignKey(
        verbose_name=_('Assigned Object Type'),
        to=ContentType,
        limit_choices_to=BGPROUTER_ASSIGNMENT_MODEL,
        on_delete=models.PROTECT,
        related_name='+',
        blank=True,
        null=True,
    )
    assigned_object_id = models.PositiveBigIntegerField(
        verbose_name=_('Assigned Object ID'), blank=True, null=True
    )
    assigned_object = GenericForeignKey(
        ct_field='assigned_object_type', fk_field='assigned_object_id'
    )
    asn = models.ForeignKey(
        verbose_name=_('ASN'),
        to='ipam.ASN',
        on_delete=models.PROTECT,
        related_name='router',
    )
    settings = GenericRelation(
        verbose_name=_('Settings'),
        to='netbox_routing.BGPSetting',
        related_name='router',
        related_query_name='router',
        content_type_field='assigned_object_type',
        object_id_field='assigned_object_id',
    )
    tenant = models.ForeignKey(
        verbose_name=_('Tenant'),
        to='tenancy.Tenant',
        on_delete=models.PROTECT,
        related_name='bgp_routers',
        blank=True,
        null=True,
    )
    policy_templates = ManyToManyField(
        to='netbox_routing.BGPPolicyTemplate',
        related_name='routers',
        verbose_name=_('Policy Templates'),
    )
    session_templates = ManyToManyField(
        to='netbox_routing.BGPSessionTemplate',
        related_name='routers',
        verbose_name=_('Session Templates'),
    )
    peer_templates = ManyToManyField(
        to='netbox_routing.BGPPeerTemplate',
        related_name='routers',
        verbose_name=_('Peer Templates'),
    )

    clone_fields = (
        'region',
        'site_group',
        'site',
        'location',
        'device',
        'asn',
        'policy_templates',
        'session_templates',
        'peer_templates',
    )
    prerequisite_models = (
        'dcim.Device',
        'ipam.ASN',
    )

    class Meta:
        verbose_name = 'BGP Router'
        verbose_name_plural = 'BGP Routers'
        ordering = (
            'assigned_object_type',
            'assigned_object_id',
            'asn',
            'name',
        )
        constraints = [
            models.UniqueConstraint(
                fields=(
                    'assigned_object_type',
                    'assigned_object_id',
                    'asn',
                ),
                name='%(app_label)s_%(class)s_assigned_object_asn',
                nulls_distinct=False,
            ),
            models.UniqueConstraint(
                fields=(
                    'assigned_object_type',
                    'assigned_object_id',
                    'name',
                ),
                name='%(app_label)s_%(class)s_assigned_object_name',
                nulls_distinct=False,
            ),
            models.CheckConstraint(
                name='%(app_label)s_%(class)s_identifier',
                condition=models.Q(asn__isnull=False) | models.Q(name__isnull=False),
            ),
        ]

    def __str__(self):
        if self.name:
            if self.assigned_object:
                return f'{self.name} ({self.assigned_object}:{self.asn})'
            return f'{self.name} ({self.asn})'
        elif self.assigned_object:
            return f'{self.assigned_object}:{self.asn}'
        return f'{self.asn}'

    def get_absolute_url(self):
        return reverse('plugins:netbox_routing:bgprouter', args=[self.pk])


class BGPScope(PrimaryModel):
    router = models.ForeignKey(
        verbose_name=_('Router'),
        to=BGPRouter,
        on_delete=models.PROTECT,
        related_name='scopes',
        blank=False,
        null=False,
    )
    vrf = models.ForeignKey(
        verbose_name=_('VRF'),
        to='ipam.VRF',
        on_delete=models.PROTECT,
        related_name='scopes',
        blank=True,
        null=True,
    )
    tenant = models.ForeignKey(
        verbose_name=_('Tenant'),
        to='tenancy.Tenant',
        on_delete=models.PROTECT,
        related_name='bgp_scopes',
        blank=True,
        null=True,
    )

    settings = GenericRelation(
        verbose_name=_('Settings'),
        to='netbox_routing.BGPSetting',
        related_name='scope',
        related_query_name='scope',
        content_type_field='assigned_object_type',
        object_id_field='assigned_object_id',
    )

    clone_fields = (
        'router',
        'vrf',
    )
    prerequisite_models = ('netbox_routing.BGPRouter',)

    class Meta:
        verbose_name = 'BGP Scope'
        verbose_name_plural = 'BGP Scopes'
        ordering = ('router', 'vrf')
        constraints = [
            models.UniqueConstraint(
                fields=('router', 'vrf'),
                name='%(app_label)s_%(class)s_router_vrf',
                nulls_distinct=False,
            ),
        ]

    def __str__(self):
        return f'{self.router}: {self.vrf or "Global VRF"}'

    def get_absolute_url(self):
        return reverse('plugins:netbox_routing:bgpscope', args=[self.pk])


class BGPAddressFamily(PrimaryModel):
    scope = models.ForeignKey(
        verbose_name=_('Scope'),
        to='netbox_routing.BGPScope',
        on_delete=models.PROTECT,
        related_name='address_families',
    )
    address_family = models.CharField(
        verbose_name=_('Address Family'), max_length=50, choices=BGPAddressFamilyChoices
    )
    tenant = models.ForeignKey(
        verbose_name=_('Tenant'),
        to='tenancy.Tenant',
        on_delete=models.PROTECT,
        related_name='bgp_address_families',
        blank=True,
        null=True,
    )
    settings = GenericRelation(
        verbose_name=_('Settings'),
        to='netbox_routing.BGPSetting',
        related_name='address_family',
        related_query_name='address_family',
        content_type_field='assigned_object_type',
        object_id_field='assigned_object_id',
    )

    clone_fields = (
        'scope',
        'address_family',
    )
    prerequisite_models = ('netbox_routing.BGPScope',)

    class Meta:
        verbose_name = 'BGP Address Family'
        verbose_name_plural = 'BGP Address Families'
        ordering = ('scope', 'address_family')
        constraints = [
            models.UniqueConstraint(
                fields=('scope', 'address_family'),
                name='%(app_label)s_%(class)s_scope_address_family',
            ),
        ]

    def __str__(self):
        return f'{self.scope} ({self.get_address_family_display()})'

    def get_absolute_url(self):
        return reverse('plugins:netbox_routing:bgpaddressfamily', args=[self.pk])


class BGPPeer(PrimaryModel):
    name = models.CharField(
        verbose_name=_('Name'),
        max_length=100,
        blank=True,
        null=True,
    )
    scope = models.ForeignKey(
        verbose_name=_('Scope'),
        to='netbox_routing.BGPScope',
        on_delete=models.PROTECT,
        related_name='peers',
        blank=True,
        null=True,
    )
    peer = models.ForeignKey(
        verbose_name=_('Peer Address'),
        to='ipam.IPAddress',
        on_delete=models.PROTECT,
        related_name='remote_peers',
        blank=False,
        null=False,
    )
    source = models.ForeignKey(
        verbose_name=_('Source Address'),
        to='ipam.IPAddress',
        on_delete=models.PROTECT,
        related_name='source_peers',
        blank=True,
        null=True,
    )
    peer_group = models.ForeignKey(
        verbose_name=_('Peer Group'),
        to='netbox_routing.BGPPeerTemplate',
        on_delete=models.PROTECT,
        related_name='peers',
        blank=True,
        null=True,
    )
    peer_session = models.ForeignKey(
        verbose_name=_('Peer Session'),
        to='netbox_routing.BGPSessionTemplate',
        on_delete=models.PROTECT,
        related_name='peers',
        blank=True,
        null=True,
    )
    remote_as = models.ForeignKey(
        verbose_name=_('Remote AS'),
        to='ipam.ASN',
        on_delete=models.PROTECT,
        related_name='+',
        blank=True,
        null=True,
    )
    enabled = models.BooleanField(verbose_name=_('Enabled'), blank=True, null=True)
    status = models.CharField(
        verbose_name=_('Status'),
        max_length=10,
        choices=BGPStatusChoices,
        null=True,
        blank=True,
    )
    local_as = models.ForeignKey(
        verbose_name=_('Local AS'),
        to='ipam.ASN',
        on_delete=models.PROTECT,
        related_name='+',
        blank=True,
        null=True,
    )
    bfd = models.ForeignKey(
        verbose_name=_('BFD PRofile'),
        to='netbox_routing.BFDProfile',
        on_delete=models.PROTECT,
        related_name='bgp_peers',
        blank=True,
        null=True,
    )
    ttl = models.PositiveSmallIntegerField(
        verbose_name=_('TTL'),
        blank=True,
        null=True,
        validators=[MinValueValidator(0), MaxValueValidator(255)],
    )
    password = models.CharField(
        verbose_name=_('Password'), max_length=255, blank=True, null=True
    )
    tenant = models.ForeignKey(
        verbose_name=_('Tenant'),
        to='tenancy.Tenant',
        on_delete=models.PROTECT,
        related_name='bgp_peers',
        blank=True,
        null=True,
    )
    address_families = GenericRelation(
        verbose_name=_('Address Families'),
        to='netbox_routing.BGPPeerAddressFamily',
        content_type_field='assigned_object_type',
        object_id_field='assigned_object_id',
        related_query_name='peer',
    )
    settings = GenericRelation(
        verbose_name=_('Settings'),
        to='netbox_routing.BGPSetting',
        related_name='peer',
        related_query_name='peer',
        content_type_field='assigned_object_type',
        object_id_field='assigned_object_id',
    )

    clone_fields = (
        'scope',
        'remote_as',
        'local_as',
        'enabled',
        'bfd',
        'password',
    )
    prerequisite_models = (
        'netbox_routing.BGPRouter',
        'netbox_routing.BGPScope',
    )

    class Meta:
        verbose_name = 'BGP Peer'
        verbose_name_plural = 'BGP Peers'
        ordering = (
            'scope',
            'peer',
            'name',
        )
        constraints = [
            models.UniqueConstraint(
                fields=('scope', 'peer', 'name'),
                name='%(app_label)s_%(class)s_scope_peer_name',
                nulls_distinct=False,
            ),
        ]

    def __str__(self):
        if self.remote_as:
            return f'{self.peer} ({self.remote_as})'
        return f'{self.peer}'

    def get_absolute_url(self):
        from django.urls import reverse

        return reverse('plugins:netbox_routing:bgppeer', args=[self.pk])


class BGPPeerAddressFamily(SearchAttributeMixin, PrimaryModel):
    assigned_object_type = models.ForeignKey(
        verbose_name=_('Assigned Object Type'),
        to=ContentType,
        limit_choices_to=BGPPEERAF_ASSIGNMENT_MODELS,
        on_delete=models.PROTECT,
        related_name='+',
        blank=True,
        null=True,
    )
    assigned_object_id = models.PositiveBigIntegerField(
        verbose_name=_('Assigned Object ID'), blank=True, null=True
    )
    assigned_object = GenericForeignKey(
        ct_field='assigned_object_type', fk_field='assigned_object_id'
    )
    address_family = models.ForeignKey(
        verbose_name=_('Address Family'),
        to='netbox_routing.BGPAddressFamily',
        on_delete=models.PROTECT,
        related_name='peer_address_families',
    )
    peer_policy = models.ForeignKey(
        verbose_name=_('Peer Policy'),
        to='netbox_routing.BGPPolicyTemplate',
        on_delete=models.PROTECT,
        related_name='peer_address_families',
        blank=True,
        null=True,
    )
    enabled = models.BooleanField(verbose_name=_('Enabled'), blank=True, null=True)

    prefixlist_out = models.ForeignKey(
        verbose_name=_('Outbound Prefix List'),
        to='netbox_routing.PrefixList',
        on_delete=models.PROTECT,
        related_name='peer_address_families_out',
        blank=True,
        null=True,
    )
    prefixlist_in = models.ForeignKey(
        verbose_name=_('Inbound Prefix List'),
        to='netbox_routing.PrefixList',
        on_delete=models.PROTECT,
        related_name='peer_address_families_in',
        blank=True,
        null=True,
    )
    routemap_out = models.ForeignKey(
        verbose_name=_('Outbound Route Map'),
        to='netbox_routing.RouteMap',
        on_delete=models.PROTECT,
        related_name='peer_address_families_out',
        blank=True,
        null=True,
    )
    routemap_in = models.ForeignKey(
        verbose_name=_('Inbound Route Map'),
        to='netbox_routing.RouteMap',
        on_delete=models.PROTECT,
        related_name='peer_address_families_in',
        blank=True,
        null=True,
    )
    tenant = models.ForeignKey(
        verbose_name=_('Tenant'),
        to='tenancy.Tenant',
        on_delete=models.PROTECT,
        related_name='bgp_peer_address_families',
        blank=True,
        null=True,
    )
    settings = GenericRelation(
        verbose_name=_('Settings'),
        to='netbox_routing.BGPSetting',
        related_name='peer_address_families',
        related_query_name='peer_address_families',
        content_type_field='assigned_object_type',
        object_id_field='assigned_object_id',
    )

    clone_fields = (
        'address_family',
        'peer_policy',
        'enabled',
        'prefixlist_out',
        'prefixlist_in',
        'routemap_out',
        'routemap_in',
        'tenant',
    )
    prerequisite_models = (
        'netbox_routing.BGPRouter',
        'netbox_routing.BGPScope',
    )

    class Meta:
        verbose_name = 'BGP Peer Address Family'
        verbose_name_plural = 'BGP Peer Address Families'
        ordering = ('assigned_object_type', 'assigned_object_id', 'address_family')
        constraints = [
            models.UniqueConstraint(
                fields=('assigned_object_type', 'assigned_object_id', 'address_family'),
                name='%(app_label)s_%(class)s_assigned_object_address_family',
            ),
        ]

    def __str__(self):
        return f'{self.assigned_object} ({self.address_family})'


class BFDProfile(PrimaryModel):
    name = models.CharField(verbose_name=_('Name'), max_length=100)
    min_tx_int = models.PositiveIntegerField(
        verbose_name=_('Min TX Interval'),
        validators=[MinValueValidator(60), MaxValueValidator(60000)],
    )
    min_rx_int = models.PositiveIntegerField(
        verbose_name=_('Min RX Interval'),
        validators=[MinValueValidator(60), MaxValueValidator(60000)],
    )
    multiplier = models.PositiveSmallIntegerField(
        verbose_name=_('Multiplier'),
        validators=[MinValueValidator(0), MaxValueValidator(255)],
    )
    hold = models.PositiveIntegerField(
        verbose_name=_('Hold Time'),
        blank=True,
        null=True,
        validators=[MinValueValidator(60), MaxValueValidator(60000)],
    )
    tenant = models.ForeignKey(
        verbose_name=_('Tenant'),
        to='tenancy.Tenant',
        on_delete=models.PROTECT,
        related_name='bfd_profiles',
        blank=True,
        null=True,
    )

    clone_fields = ('tenant',)
    prerequisite_models = (
        'netbox_routing.BGPRouter',
        'netbox_routing.BGPScope',
    )

    class Meta:
        verbose_name = 'BFD Profile'
        verbose_name_plural = 'BFD Profiles'
        ordering = ('name',)
        constraints = [
            models.UniqueConstraint(
                fields=('name',),
                name='%(app_label)s_%(class)s_name',
            ),
        ]

    def __str__(self):
        return f'{self.name}'



# Generated by Django 4.2.4 on 2023-12-19 17:18

from django.db import migrations, models
import django.db.models.deletion
import netbox_routing.fields.ip
import taggit.managers
import utilities.json


class Migration(migrations.Migration):

    dependencies = [
        ('netbox_routing', '0005_ospf'),
    ]

    operations = [
        migrations.CreateModel(
            name='BGPAddressFamily',
            fields=[
                (
                    'id',
                    models.BigAutoField(
                        auto_created=True, primary_key=True, serialize=False
                    ),
                ),
                ('created', models.DateTimeField(auto_now_add=True, null=True)),
                ('last_updated', models.DateTimeField(auto_now=True, null=True)),
                (
                    'custom_field_data',
                    models.JSONField(
                        blank=True,
                        default=dict,
                        encoder=utilities.json.CustomFieldJSONEncoder,
                    ),
                ),
                ('address_family', models.CharField(max_length=50)),
            ],
            options={
                'verbose_name': 'BGP Address Family',
            },
        ),
        migrations.CreateModel(
            name='BGPRouter',
            fields=[
                (
                    'id',
                    models.BigAutoField(
                        auto_created=True, primary_key=True, serialize=False
                    ),
                ),
                ('created', models.DateTimeField(auto_now_add=True, null=True)),
                ('last_updated', models.DateTimeField(auto_now=True, null=True)),
                (
                    'custom_field_data',
                    models.JSONField(
                        blank=True,
                        default=dict,
                        encoder=utilities.json.CustomFieldJSONEncoder,
                    ),
                ),
                (
                    'asn',
                    models.ForeignKey(
                        on_delete=django.db.models.deletion.PROTECT,
                        related_name='router',
                        to='ipam.asn',
                    ),
                ),
                (
                    'device',
                    models.ForeignKey(
                        on_delete=django.db.models.deletion.PROTECT,
                        related_name='router',
                        to='dcim.device',
                    ),
                ),
                (
                    'tags',
                    taggit.managers.TaggableManager(
                        through='extras.TaggedItem', to='extras.Tag'
                    ),
                ),
            ],
            options={
                'verbose_name': 'BGP Router',
            },
        ),
        migrations.AddField(
            model_name='routemapentry',
            name='match',
            field=models.JSONField(blank=True, null=True),
        ),
        migrations.AddField(
            model_name='routemapentry',
            name='set',
            field=models.JSONField(blank=True, null=True),
        ),
        migrations.CreateModel(
            name='BGPSetting',
            fields=[
                (
                    'id',
                    models.BigAutoField(
                        auto_created=True, primary_key=True, serialize=False
                    ),
                ),
                ('created', models.DateTimeField(auto_now_add=True, null=True)),
                ('last_updated', models.DateTimeField(auto_now=True, null=True)),
                (
                    'custom_field_data',
                    models.JSONField(
                        blank=True,
                        default=dict,
                        encoder=utilities.json.CustomFieldJSONEncoder,
                    ),
                ),
                (
                    'assigned_object_id',
                    models.PositiveBigIntegerField(blank=True, null=True),
                ),
                ('key', models.CharField()),
                ('value', models.CharField()),
                (
                    'assigned_object_type',
                    models.ForeignKey(
                        blank=True,
                        limit_choices_to=models.Q(
                            models.Q(
                                models.Q(
                                    ('app_label', 'netbox_routing'),
                                    ('model', 'bgprouter'),
                                ),
                                models.Q(
                                    ('app_label', 'netbox_routing'),
                                    ('model', 'bgpscope'),
                                ),
                                _connector='OR',
                            )
                        ),
                        null=True,
                        on_delete=django.db.models.deletion.PROTECT,
                        related_name='+',
                        to='contenttypes.contenttype',
                    ),
                ),
                (
                    'tags',
                    taggit.managers.TaggableManager(
                        through='extras.TaggedItem', to='extras.Tag'
                    ),
                ),
            ],
            options={
                'verbose_name': 'BGP Setting',
            },
        ),
        migrations.CreateModel(
            name='BGPSessionTemplate',
            fields=[
                (
                    'id',
                    models.BigAutoField(
                        auto_created=True, primary_key=True, serialize=False
                    ),
                ),
                ('created', models.DateTimeField(auto_now_add=True, null=True)),
                ('last_updated', models.DateTimeField(auto_now=True, null=True)),
                (
                    'custom_field_data',
                    models.JSONField(
                        blank=True,
                        default=dict,
                        encoder=utilities.json.CustomFieldJSONEncoder,
                    ),
                ),
                ('name', models.CharField(max_length=255)),
                ('enabled', models.BooleanField(blank=True, null=True)),
                (
                    'router',
                    models.ForeignKey(
                        on_delete=django.db.models.deletion.PROTECT,
                        related_name='session_templates',
                        to='netbox_routing.bgprouter',
                    ),
                ),
                (
                    'tags',
                    taggit.managers.TaggableManager(
                        through='extras.TaggedItem', to='extras.Tag'
                    ),
                ),
            ],
            options={
                'abstract': False,
            },
        ),
        migrations.CreateModel(
            name='BGPScope',
            fields=[
                (
                    'id',
                    models.BigAutoField(
                        auto_created=True, primary_key=True, serialize=False
                    ),
                ),
                ('created', models.DateTimeField(auto_now_add=True, null=True)),
                ('last_updated', models.DateTimeField(auto_now=True, null=True)),
                (
                    'custom_field_data',
                    models.JSONField(
                        blank=True,
                        default=dict,
                        encoder=utilities.json.CustomFieldJSONEncoder,
                    ),
                ),
                (
                    'router',
                    models.ForeignKey(
                        on_delete=django.db.models.deletion.PROTECT,
                        related_name='scopes',
                        to='netbox_routing.bgprouter',
                    ),
                ),
                (
                    'tags',
                    taggit.managers.TaggableManager(
                        through='extras.TaggedItem', to='extras.Tag'
                    ),
                ),
                (
                    'vrf',
                    models.ForeignKey(
                        blank=True,
                        null=True,
                        on_delete=django.db.models.deletion.PROTECT,
                        related_name='scopes',
                        to='ipam.vrf',
                    ),
                ),
            ],
            options={
                'verbose_name': 'BGP Scope',
            },
        ),
        migrations.CreateModel(
            name='BGPPoliyTemplate',
            fields=[
                (
                    'id',
                    models.BigAutoField(
                        auto_created=True, primary_key=True, serialize=False
                    ),
                ),
                ('created', models.DateTimeField(auto_now_add=True, null=True)),
                ('last_updated', models.DateTimeField(auto_now=True, null=True)),
                (
                    'custom_field_data',
                    models.JSONField(
                        blank=True,
                        default=dict,
                        encoder=utilities.json.CustomFieldJSONEncoder,
                    ),
                ),
                ('name', models.CharField(max_length=255)),
                ('enabled', models.BooleanField(blank=True, null=True)),
                (
                    'prefixlist_in',
                    models.ForeignKey(
                        blank=True,
                        null=True,
                        on_delete=django.db.models.deletion.PROTECT,
                        related_name='template_afs_in',
                        to='netbox_routing.prefixlist',
                    ),
                ),
                (
                    'prefixlist_out',
                    models.ForeignKey(
                        blank=True,
                        null=True,
                        on_delete=django.db.models.deletion.PROTECT,
                        related_name='template_afs_out',
                        to='netbox_routing.prefixlist',
                    ),
                ),
                (
                    'routemap_in',
                    models.ForeignKey(
                        blank=True,
                        null=True,
                        on_delete=django.db.models.deletion.PROTECT,
                        related_name='template_afs_in',
                        to='netbox_routing.routemap',
                    ),
                ),
                (
                    'routemap_out',
                    models.ForeignKey(
                        blank=True,
                        null=True,
                        on_delete=django.db.models.deletion.PROTECT,
                        related_name='template_afs_out',
                        to='netbox_routing.routemap',
                    ),
                ),
                (
                    'router',
                    models.ForeignKey(
                        on_delete=django.db.models.deletion.PROTECT,
                        related_name='policy_templates',
                        to='netbox_routing.bgprouter',
                    ),
                ),
                (
                    'tags',
                    taggit.managers.TaggableManager(
                        through='extras.TaggedItem', to='extras.Tag'
                    ),
                ),
            ],
            options={
                'abstract': False,
            },
        ),
        migrations.CreateModel(
            name='BGPPeerTemplate',
            fields=[
                (
                    'id',
                    models.BigAutoField(
                        auto_created=True, primary_key=True, serialize=False
                    ),
                ),
                ('created', models.DateTimeField(auto_now_add=True, null=True)),
                ('last_updated', models.DateTimeField(auto_now=True, null=True)),
                (
                    'custom_field_data',
                    models.JSONField(
                        blank=True,
                        default=dict,
                        encoder=utilities.json.CustomFieldJSONEncoder,
                    ),
                ),
                ('name', models.CharField(max_length=255)),
                ('enabled', models.BooleanField(blank=True, null=True)),
                (
                    'remote_as',
                    models.ForeignKey(
                        blank=True,
                        null=True,
                        on_delete=django.db.models.deletion.PROTECT,
                        related_name='+',
                        to='ipam.asn',
                    ),
                ),
                (
                    'tags',
                    taggit.managers.TaggableManager(
                        through='extras.TaggedItem', to='extras.Tag'
                    ),
                ),
            ],
            options={
                'abstract': False,
            },
        ),
        migrations.CreateModel(
            name='BGPPeerAddressFamily',
            fields=[
                (
                    'id',
                    models.BigAutoField(
                        auto_created=True, primary_key=True, serialize=False
                    ),
                ),
                ('created', models.DateTimeField(auto_now_add=True, null=True)),
                ('last_updated', models.DateTimeField(auto_now=True, null=True)),
                (
                    'custom_field_data',
                    models.JSONField(
                        blank=True,
                        default=dict,
                        encoder=utilities.json.CustomFieldJSONEncoder,
                    ),
                ),
                (
                    'assigned_object_id',
                    models.PositiveBigIntegerField(blank=True, null=True),
                ),
                ('enabled', models.BooleanField(blank=True, null=True)),
                (
                    'address_family',
                    models.ForeignKey(
                        on_delete=django.db.models.deletion.PROTECT,
                        related_name='address_families',
                        to='netbox_routing.bgpaddressfamily',
                    ),
                ),
                (
                    'assigned_object_type',
                    models.ForeignKey(
                        blank=True,
                        limit_choices_to=models.Q(
                            models.Q(
                                models.Q(
                                    ('app_label', 'netbox_routing'),
                                    ('model', 'bgppeer'),
                                ),
                                models.Q(
                                    ('app_label', 'netbox_routing'),
                                    ('model', 'bgppeergroup'),
                                ),
                                models.Q(
                                    ('app_label', 'netbox_routing'),
                                    ('model', 'bgptemplatepeer'),
                                ),
                                models.Q(
                                    ('app_label', 'netbox_routing'),
                                    ('model', 'bgptemplatepeerpolicy'),
                                ),
                                _connector='OR',
                            )
                        ),
                        null=True,
                        on_delete=django.db.models.deletion.PROTECT,
                        related_name='+',
                        to='contenttypes.contenttype',
                    ),
                ),
                (
                    'peer_session',
                    models.ForeignKey(
                        blank=True,
                        null=True,
                        on_delete=django.db.models.deletion.PROTECT,
                        related_name='peer_afs',
                        to='netbox_routing.bgpsessiontemplate',
                    ),
                ),
                (
                    'prefixlist_in',
                    models.ForeignKey(
                        blank=True,
                        null=True,
                        on_delete=django.db.models.deletion.PROTECT,
                        related_name='peer_afs_in',
                        to='netbox_routing.prefixlist',
                    ),
                ),
                (
                    'prefixlist_out',
                    models.ForeignKey(
                        blank=True,
                        null=True,
                        on_delete=django.db.models.deletion.PROTECT,
                        related_name='peer_afs_out',
                        to='netbox_routing.prefixlist',
                    ),
                ),
                (
                    'routemap_in',
                    models.ForeignKey(
                        blank=True,
                        null=True,
                        on_delete=django.db.models.deletion.PROTECT,
                        related_name='peer_afs_in',
                        to='netbox_routing.routemap',
                    ),
                ),
                (
                    'routemap_out',
                    models.ForeignKey(
                        blank=True,
                        null=True,
                        on_delete=django.db.models.deletion.PROTECT,
                        related_name='peer_afs_out',
                        to='netbox_routing.routemap',
                    ),
                ),
                (
                    'tags',
                    taggit.managers.TaggableManager(
                        through='extras.TaggedItem', to='extras.Tag'
                    ),
                ),
            ],
            options={
                'abstract': False,
            },
        ),
        migrations.CreateModel(
            name='BGPPeer',
            fields=[
                (
                    'id',
                    models.BigAutoField(
                        auto_created=True, primary_key=True, serialize=False
                    ),
                ),
                ('created', models.DateTimeField(auto_now_add=True, null=True)),
                ('last_updated', models.DateTimeField(auto_now=True, null=True)),
                (
                    'custom_field_data',
                    models.JSONField(
                        blank=True,
                        default=dict,
                        encoder=utilities.json.CustomFieldJSONEncoder,
                    ),
                ),
                (
                    'assigned_object_id',
                    models.PositiveBigIntegerField(blank=True, null=True),
                ),
                ('peer', netbox_routing.fields.ip.IPAddressField()),
                ('enabled', models.BooleanField(blank=True, null=True)),
                (
                    'assigned_object_type',
                    models.ForeignKey(
                        blank=True,
                        limit_choices_to=models.Q(
                            models.Q(
                                models.Q(
                                    ('app_label', 'netbox_routing'),
                                    ('model', 'bgprouter'),
                                ),
                                models.Q(
                                    ('app_label', 'netbox_routing'),
                                    ('model', 'bgpscope'),
                                ),
                                _connector='OR',
                            )
                        ),
                        null=True,
                        on_delete=django.db.models.deletion.PROTECT,
                        related_name='+',
                        to='contenttypes.contenttype',
                    ),
                ),
                (
                    'peer_group',
                    models.ForeignKey(
                        blank=True,
                        null=True,
                        on_delete=django.db.models.deletion.PROTECT,
                        related_name='peers',
                        to='netbox_routing.bgppeertemplate',
                    ),
                ),
                (
                    'peer_policy',
                    models.ForeignKey(
                        blank=True,
                        null=True,
                        on_delete=django.db.models.deletion.PROTECT,
                        related_name='peers',
                        to='netbox_routing.bgppoliytemplate',
                    ),
                ),
                (
                    'remote_as',
                    models.ForeignKey(
                        blank=True,
                        null=True,
                        on_delete=django.db.models.deletion.PROTECT,
                        related_name='+',
                        to='ipam.asn',
                    ),
                ),
                (
                    'tags',
                    taggit.managers.TaggableManager(
                        through='extras.TaggedItem', to='extras.Tag'
                    ),
                ),
            ],
            options={
                'abstract': False,
            },
        ),
        migrations.AddField(
            model_name='bgpaddressfamily',
            name='scope',
            field=models.ForeignKey(
                on_delete=django.db.models.deletion.PROTECT,
                related_name='address_families',
                to='netbox_routing.bgpscope',
            ),
        ),
        migrations.AddField(
            model_name='bgpaddressfamily',
            name='tags',
            field=taggit.managers.TaggableManager(
                through='extras.TaggedItem', to='extras.Tag'
            ),
        ),
    ]



from ipam.models import Prefix
from utilities.testing import APIViewTestCases

from netbox_routing.models.objects import *

__all__ = (
    'ASPathTestCase',
    'ASPathEntryTestCase',
    'PrefixListTestCase',
    'PrefixListEntryTestCase',
    'RouteMapTestCase',
    'RouteMapEntryTestCase',
)


class ASPathTestCase(APIViewTestCases.APIViewTestCase):
    model = ASPath
    view_namespace = "plugins-api:netbox_routing"
    graphql_base_name = 'aspath'
    brief_fields = [
        'display',
        'id',
        'name',
        'url',
    ]

    bulk_update_data = {'description': 'Description'}

    @classmethod
    def setUpTestData(cls):
        cls.aspath = (
            cls.model(
                name='AS Path List 1',
            ),
            cls.model(
                name='AS Path List 2',
            ),
            cls.model(
                name='AS Path List 3',
            ),
        )
        cls.model.objects.bulk_create(cls.aspath)

        cls.create_data = [
            {
                'name': 'AS Path List 4',
            },
            {
                'name': 'AS Path List 5',
            },
        ]


class ASPathEntryTestCase(APIViewTestCases.APIViewTestCase):
    model = ASPathEntry
    view_namespace = "plugins-api:netbox_routing"
    graphql_base_name = 'aspath_entry'
    brief_fields = [
        'action',
        'aspath',
        'display',
        'id',
        'pattern',
        'sequence',
        'url',
    ]

    bulk_update_data = {'description': 'Description'}

    @classmethod
    def setUpTestData(cls):
        cls.aspath = (
            ASPath(
                name='AS Path List 1',
            ),
            ASPath(
                name='AS Path List 2',
            ),
        )
        ASPath.objects.bulk_create(cls.aspath)

        cls.aspath_entry = (
            cls.model(
                aspath=cls.aspath[0], action='permit', sequence=1, pattern='^2448$'
            ),
            cls.model(
                aspath=cls.aspath[0], action='permit', sequence=2, pattern='^2448 2447$'
            ),
            cls.model(
                aspath=cls.aspath[1], action='permit', sequence=1, pattern='^.*$'
            ),
        )
        cls.model.objects.bulk_create(cls.aspath_entry)

        cls.create_data = [
            {
                'aspath': cls.aspath[0].pk,
                'action': 'permit',
                'sequence': 3,
                'pattern': '^2448 2448 2448$',
            },
            {
                'aspath': cls.aspath[0].pk,
                'action': 'permit',
                'sequence': 4,
                'pattern': '^2448 2448 2448 2448$',
            },
        ]


class PrefixListTestCase(APIViewTestCases.APIViewTestCase):
    model = PrefixList
    view_namespace = "plugins-api:netbox_routing"
    graphql_base_name = 'prefixlist'
    brief_fields = [
        'display',
        'id',
        'name',
        'url',
    ]

    bulk_update_data = {'description': 'Description'}

    @classmethod
    def setUpTestData(cls):
        cls.prefix_list = (
            cls.model(
                name='Prefix List 1',
            ),
            cls.model(
                name='Prefix List 2',
            ),
            cls.model(
                name='Prefix List 3',
            ),
        )
        cls.model.objects.bulk_create(cls.prefix_list)

        cls.create_data = [
            {
                'name': 'Prefix List 4',
            },
            {
                'name': 'Prefix List 5',
            },
        ]


class PrefixListEntryTestCase(APIViewTestCases.APIViewTestCase):
    model = PrefixListEntry
    view_namespace = "plugins-api:netbox_routing"
    graphql_base_name = 'prefixlist_entry'
    brief_fields = [
        'action',
        'assigned_prefix_id',
        'assigned_prefix_type',
        'display',
        'ge',
        'id',
        'le',
        'prefix_list',
        'sequence',
        'url',
    ]

    bulk_update_data = {'description': 'Description'}

    @classmethod
    def setUpTestData(cls):
        prefixes = (
            Prefix(prefix='10.0.0.0/24'),
            Prefix(prefix='10.0.1.0/24'),
            Prefix(prefix='10.0.2.0/24'),
            Prefix(prefix='10.0.3.0/24'),
        )
        Prefix.objects.bulk_create(prefixes)

        custom_prefixes = (
            CustomPrefix(prefix='10.0.0.0/24'),
            CustomPrefix(prefix='10.0.4.0/24'),
        )
        CustomPrefix.objects.bulk_create(custom_prefixes)

        cls.prefix_list = (
            PrefixList(
                name='Prefix List 1',
            ),
            PrefixList(
                name='Prefix List 2',
            ),
        )
        PrefixList.objects.bulk_create(cls.prefix_list)

        cls.prefix_list_entry = (
            cls.model(
                prefix_list=cls.prefix_list[0],
                action='permit',
                sequence=1,
                assigned_prefix=prefixes[0],
            ),
            cls.model(
                prefix_list=cls.prefix_list[0],
                action='permit',
                sequence=2,
                assigned_prefix=prefixes[1],
            ),
            cls.model(
                prefix_list=cls.prefix_list[1],
                action='permit',
                sequence=1,
                assigned_prefix=custom_prefixes[0],
            ),
        )
        cls.model.objects.bulk_create(cls.prefix_list_entry)

        cls.create_data = [
            {
                'prefix_list': cls.prefix_list[0].pk,
                'action': 'permit',
                'sequence': 3,
                'assigned_prefix_type': 'ipam.prefix',
                'assigned_prefix_id': prefixes[0].pk,
            },
            {
                'prefix_list': cls.prefix_list[0].pk,
                'action': 'permit',
                'sequence': 4,
                'assigned_prefix_type': 'netbox_routing.customprefix',
                'assigned_prefix_id': custom_prefixes[1].pk,
            },
        ]


class RouteMapTestCase(APIViewTestCases.APIViewTestCase):
    model = RouteMap
    view_namespace = "plugins-api:netbox_routing"
    brief_fields = [
        'display',
        'id',
        'name',
        'url',
    ]

    bulk_update_data = {'description': 'Description'}

    @classmethod
    def setUpTestData(cls):
        cls.route_map = (
            cls.model(
                name='Route Map 1',
            ),
            cls.model(
                name='Route Map 2',
            ),
            cls.model(
                name='Route Map 3',
            ),
        )
        cls.model.objects.bulk_create(cls.route_map)

        cls.create_data = [
            {
                'name': 'Route Map 4',
            },
            {
                'name': 'Route Map 5',
            },
        ]


class RouteMapEntryTestCase(APIViewTestCases.APIViewTestCase):
    model = RouteMapEntry
    view_namespace = "plugins-api:netbox_routing"
    brief_fields = [
        'action',
        'display',
        'id',
        'route_map',
        'sequence',
        'url',
    ]

    bulk_update_data = {'description': 'Description'}

    @classmethod
    def setUpTestData(cls):
        cls.route_map = (
            RouteMap(
                name='Route Map 1',
            ),
            RouteMap(
                name='Route Map 2',
            ),
        )
        RouteMap.objects.bulk_create(cls.route_map)

        cls.route_map_entry = (
            cls.model(
                route_map=cls.route_map[0],
                action='permit',
                sequence=1,
                match={'tags': 1},
                set={},
            ),
            cls.model(
                route_map=cls.route_map[0],
                action='permit',
                sequence=2,
                match={'tags': 2},
                set={},
            ),
            cls.model(
                route_map=cls.route_map[1],
                action='permit',
                sequence=1,
                match={'tags': 3},
                set={},
            ),
        )
        cls.model.objects.bulk_create(cls.route_map_entry)

        cls.create_data = [
            {
                'route_map': cls.route_map[0].pk,
                'action': 'permit',
                'sequence': 3,
                'match': {'tags': 4},
            },
            {
                'route_map': cls.route_map[0].pk,
                'action': 'permit',
                'sequence': 4,
                'match': {'tags': 5},
            },
        ]



from netbox.views.generic import (
    ObjectListView,
    ObjectView,
    ObjectEditView,
    ObjectDeleteView,
    ObjectChildrenView,
    BulkEditView,
    BulkDeleteView,
)
from utilities.views import register_model_view, ViewTab

from netbox_routing.filtersets.community import *
from netbox_routing.forms.community import *
from netbox_routing.models.community import *
from netbox_routing.tables.community import *

__all__ = (
    'CommunityListListView',
    'CommunityListDetailView',
    'CommunityListEditView',
    'CommunityListDeleteView',
    'CommunityListEntriesView',
    'CommunityListBulkEditView',
    'CommunityListBulkDeleteView',
    'CommunityListView',
    'CommunityDetailView',
    'CommunityEditView',
    'CommunityDeleteView',
    'CommunityBulkEditView',
    'CommunityBulkDeleteView',
    'CommunityListEntryListView',
    'CommunityListEntryDetailView',
    'CommunityListEntryEditView',
    'CommunityListEntryDeleteView',
    'CommunityListEntryBulkEditView',
    'CommunityListEntryBulkDeleteView',
)


#
# Prefix List
#
@register_model_view(CommunityList, name='list', path='', detail=False)
class CommunityListListView(ObjectListView):
    queryset = CommunityList.objects.all()
    table = CommunityListTable
    filterset = CommunityListFilterSet
    filterset_form = CommunityListFilterForm


@register_model_view(CommunityList)
class CommunityListDetailView(ObjectView):
    queryset = CommunityList.objects.all()
    template_name = 'netbox_routing/communitylist.html'


@register_model_view(CommunityList, name='add', detail=False)
@register_model_view(CommunityList, name='edit')
class CommunityListEditView(ObjectEditView):
    queryset = CommunityList.objects.all()
    form = CommunityListForm


@register_model_view(CommunityList, name='delete')
class CommunityListDeleteView(ObjectDeleteView):
    queryset = CommunityList.objects.all()


@register_model_view(CommunityList, name='entries')
class CommunityListEntriesView(ObjectChildrenView):
    queryset = CommunityList.objects.all()
    child_model = CommunityListEntry
    table = CommunityListEntryTable
    filterset = CommunityListEntryFilterSet
    filterset_form = CommunityListEntryFilterForm
    actions = ObjectChildrenView.actions
    tab = ViewTab(
        label='Entries',
        badge=lambda obj: CommunityListEntriesView.child_model.objects.filter(
            community_list=obj
        ).count(),
    )

    def get_children(self, request, parent):
        return self.child_model.objects.filter(community_list=parent)


@register_model_view(CommunityList, name='bulk_edit', detail=False)
class CommunityListBulkEditView(BulkEditView):
    queryset = CommunityList.objects.all()
    filterset = CommunityListFilterForm
    form = CommunityListBulkEditForm
    table = CommunityListTable


@register_model_view(CommunityList, name='bulk_delete', detail=False)
class CommunityListBulkDeleteView(BulkDeleteView):
    queryset = CommunityList.objects.all()
    filterset = CommunityListFilterForm
    table = CommunityListTable


#
# Community
#


@register_model_view(Community, name='list', path='', detail=False)
class CommunityListView(ObjectListView):
    queryset = Community.objects.all()
    table = CommunityTable
    filterset = CommunityFilterSet
    filterset_form = CommunityFilterForm


@register_model_view(Community)
class CommunityDetailView(ObjectView):
    queryset = Community.objects.all()
    template_name = 'netbox_routing/community.html'


@register_model_view(Community, name='add', detail=False)
@register_model_view(Community, name='edit')
class CommunityEditView(ObjectEditView):
    queryset = Community.objects.all()
    form = CommunityForm


@register_model_view(Community, name='delete')
class CommunityDeleteView(ObjectDeleteView):
    queryset = Community.objects.all()


@register_model_view(Community, name='bulk_edit', detail=False)
class CommunityBulkEditView(BulkEditView):
    queryset = Community.objects.all()
    filterset = CommunityFilterForm
    form = CommunityBulkEditForm
    table = CommunityTable


@register_model_view(Community, name='bulk_delete', detail=False)
class CommunityBulkDeleteView(BulkDeleteView):
    queryset = Community.objects.all()
    filterset = CommunityFilterForm
    table = CommunityTable


#
# Community List Entry
#


@register_model_view(CommunityListEntry, name='list', path='', detail=False)
class CommunityListEntryListView(ObjectListView):
    queryset = CommunityListEntry.objects.all()
    table = CommunityListEntryTable
    filterset = CommunityListEntryFilterSet
    filterset_form = CommunityListEntryFilterForm


@register_model_view(CommunityListEntry)
class CommunityListEntryDetailView(ObjectView):
    queryset = CommunityListEntry.objects.all()
    template_name = 'netbox_routing/communitylistentry.html'


@register_model_view(CommunityListEntry, name='add', detail=False)
@register_model_view(CommunityListEntry, name='edit')
class CommunityListEntryEditView(ObjectEditView):
    queryset = CommunityListEntry.objects.all()
    form = CommunityListEntryForm


@register_model_view(CommunityListEntry, name='delete')
class CommunityListEntryDeleteView(ObjectDeleteView):
    queryset = CommunityListEntry.objects.all()


@register_model_view(CommunityListEntry, name='bulk_edit', detail=False)
class CommunityListEntryBulkEditView(BulkEditView):
    queryset = CommunityListEntry.objects.all()
    filterset = CommunityListEntryFilterForm
    form = CommunityListEntryBulkEditForm
    table = CommunityListEntryTable


@register_model_view(CommunityListEntry, name='bulk_delete', detail=False)
class CommunityListEntryBulkDeleteView(BulkDeleteView):
    queryset = CommunityListEntry.objects.all()
    filterset = CommunityListEntryFilterForm
    table = CommunityListEntryTable
